@@ -128,12 +128,13 @@ axiom_element_create(
         uri = axiom_namespace_get_uri(ns, env);
         prefix = axiom_namespace_get_prefix(ns, env);
 
+        /*
         if (prefix && axutil_strcmp(prefix, "") == 0)
         {
             element->ns = NULL;
             return element;
         }
-
+        */
         element->ns = axiom_element_find_namespace(element,
                                                    env, *node, uri, prefix);
 
@@ -757,6 +758,7 @@ axiom_element_serialize_start_part(
     AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
     AXIS2_PARAM_CHECK(env->error, om_output, AXIS2_FAILURE);
 
+
     if (om_element->is_empty)
     {
         if (om_element->ns)
@@ -1102,19 +1104,21 @@ axiom_element_get_first_child_with_qname(
 {
     axiom_node_t *om_node = NULL;
     axiom_children_qname_iterator_t *children_iterator = NULL;
-    AXIS2_ENV_CHECK(env, NULL);
+
     AXIS2_PARAM_CHECK(env->error, element_qname, NULL);
     AXIS2_PARAM_CHECK(env->error, element_node, NULL);
 
     om_node = axiom_node_get_first_child(element_node, env);
     if (!om_node)
     {
+        AXIS2_LOG_DEBUG(env->log, AXIS2_LOG_SI, "There are no child elements for the node");
         return NULL;
     }
     children_iterator = axiom_children_qname_iterator_create(env, om_node, 
                                                              element_qname);
     if (!children_iterator)
     {
+        AXIS2_LOG_DEBUG(env->log, AXIS2_LOG_SI, "Could not create children qname iterator");
         return NULL;
     }
     om_node = NULL;
@@ -1132,7 +1136,9 @@ axiom_element_get_first_child_with_qname(
         }
         return (axiom_element_t *) axiom_node_get_data_element(om_node, env);
     }
+
     axiom_children_qname_iterator_free(children_iterator, env);
+
     return NULL;
 }
 
@@ -1442,7 +1448,7 @@ AXIS2_EXTERN axiom_namespace_t *AXIS2_CALL
 axiom_element_find_namespace_uri(
     axiom_element_t * om_element,
     const axutil_env_t * env,
-    axis2_char_t * prefix,
+    const axis2_char_t * prefix,
     axiom_node_t * element_node)
 {
     axiom_node_t *parent_node = NULL;
@@ -1688,6 +1694,7 @@ axiom_element_create_str(
     element->children_qname_iter = NULL;
     element->text_value = NULL;
     element->next_ns_prefix_number = 0;
+	element->is_empty = AXIS2_FALSE;
 
     element->localname = axutil_string_clone(localname, env);
     if (!element->localname)
