@@ -29,7 +29,7 @@
 struct axiom_node
 {
 
-    /** document only availble if build through builder */
+    /** document only available if build through builder */
     struct axiom_document *om_doc;
 
     axiom_stax_builder_t *builder;
@@ -91,6 +91,52 @@ axiom_node_create(
     return node;
 }
 
+AXIS2_EXTERN axiom_node_t* AXIS2_CALL
+axiom_node_create_from_buffer(
+    const axutil_env_t * env,
+    axis2_char_t *buffer)
+{
+    axiom_xml_reader_t *reader = NULL;
+    axiom_stax_builder_t *builder = NULL;
+    axiom_document_t *document = NULL;
+    axiom_node_t *om_node = NULL;
+
+    reader = axiom_xml_reader_create_for_memory (env, buffer, axutil_strlen (buffer),
+        "UTF-8", AXIS2_XML_PARSER_TYPE_BUFFER);
+
+    if (!reader)
+    {
+        return NULL;
+    }
+
+    builder = axiom_stax_builder_create (env, reader);
+
+    if (!builder)
+    {
+        return NULL;
+    }
+    document = axiom_stax_builder_get_document (builder, env);
+    if (!document)
+    {
+        AXIS2_LOG_ERROR (env->log, AXIS2_LOG_SI, "Document is null for deserialization");
+        return NULL;
+    }
+
+    om_node = axiom_document_get_root_element (document, env);
+
+    if (!om_node)
+    {
+        AXIS2_LOG_ERROR (env->log, AXIS2_LOG_SI, "Root element of the document is not found");
+        return NULL;
+    }
+    axiom_document_build_all (document, env);
+
+    axiom_stax_builder_free_self (builder, env);
+
+    return om_node;
+}
+
+
 static void
 axiom_node_free_detached_subtree(
     axiom_node_t * om_node,
@@ -367,7 +413,7 @@ axiom_node_insert_sibling_after(
 
     if (!om_node->parent)
     {
-        /* We shouldn't add a sibling becuase this node doesn't has a parent. 
+        /* We shouldn't add a sibling because this node doesn't has a parent. 
          * This can be the root node of the tree*/        
         return AXIS2_FAILURE;
     }
@@ -402,7 +448,7 @@ axiom_node_insert_sibling_before(
 
     if (!om_node->parent)
     {
-        /* We shouldn't add a sibling becuase this node doesn't has a parent. 
+        /* We shouldn't add a sibling because this node doesn't has a parent. 
          * This can be the root node of the tree*/
         return AXIS2_FAILURE;
     }
@@ -626,6 +672,7 @@ axiom_node_serialize_sub_tree(
     axiom_node_t *nodes[256];
     int count = 0;
     axutil_hash_t *namespaces = NULL;
+    axutil_hash_t *namespaces_from_parents = NULL;
 
     AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
 
@@ -635,6 +682,21 @@ axiom_node_serialize_sub_tree(
     }
 
     namespaces = axutil_hash_make(env);
+    if(!namespaces)
+    {
+        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                "hash for namespaces creation failed");
+        return AXIS2_FAILURE;
+    }
+
+    namespaces_from_parents = axutil_hash_make(env);
+    if(!namespaces_from_parents)
+    {
+        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                "hash for namespaces_from_parents creation failed");
+        return AXIS2_FAILURE;
+    }
+
     nodes[count++] = om_node;
 
     AXIS2_PARAM_CHECK(env->error, om_output, AXIS2_FAILURE);
@@ -663,8 +725,8 @@ axiom_node_serialize_sub_tree(
                     axutil_hash_t *new_hash = NULL;
                     new_hash =
                         axutil_hash_overlay(temp_namespaces, env, namespaces);
-                    /*if(namespaces)
-                      axutil_hash_free(namespaces, env);*/
+                    if(namespaces)
+                        axutil_hash_free(namespaces, env);
                     namespaces = new_hash;
                 }
                 namespace = axiom_element_get_namespace((axiom_element_t
@@ -682,10 +744,15 @@ axiom_node_serialize_sub_tree(
                                              AXIS2_HASH_KEY_STRING);
                         if (!ns)
                         {
-                            axiom_namespace_serialize(namespace, env,
-                                                      om_output);
-                            axutil_hash_set(namespaces, prefix,
-                                            AXIS2_HASH_KEY_STRING, namespace);
+                            ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                    AXIS2_HASH_KEY_STRING);
+                            if (!ns)
+                            {
+                                axiom_namespace_serialize(namespace, env,
+                                                          om_output);
+                                axutil_hash_set(namespaces_from_parents, prefix,
+                                                AXIS2_HASH_KEY_STRING, namespace);
+                            }
                         }
                     }
                 }
@@ -719,12 +786,15 @@ axiom_node_serialize_sub_tree(
                                                          AXIS2_HASH_KEY_STRING);
                                     if (!ns)
                                     {
-                                        axiom_namespace_serialize(namespace,
-                                                                  env,
-                                                                  om_output);
-                                        axutil_hash_set(namespaces, prefix,
-                                                        AXIS2_HASH_KEY_STRING,
-                                                        namespace);
+                                        ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                                AXIS2_HASH_KEY_STRING);
+                                        if (!ns)
+                                        {
+                                            axiom_namespace_serialize(namespace, env,
+                                                                      om_output);
+                                            axutil_hash_set(namespaces_from_parents, prefix,
+                                                            AXIS2_HASH_KEY_STRING, namespace);
+                                        }
                                     }
                                 }
                             }
@@ -738,7 +808,7 @@ axiom_node_serialize_sub_tree(
             }
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
         else if (om_node->node_type == AXIOM_DATA_SOURCE)
@@ -752,7 +822,7 @@ axiom_node_serialize_sub_tree(
             }
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
         else if (om_node->node_type == AXIOM_TEXT)
@@ -766,7 +836,7 @@ axiom_node_serialize_sub_tree(
             }
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
         else if (om_node->node_type == AXIOM_COMMENT)
@@ -780,7 +850,7 @@ axiom_node_serialize_sub_tree(
             }
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
         else if (om_node->node_type == AXIOM_DOCTYPE)
@@ -794,7 +864,7 @@ axiom_node_serialize_sub_tree(
             }
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
         else if (om_node->node_type == AXIOM_PROCESSING_INSTRUCTION)
@@ -807,7 +877,7 @@ axiom_node_serialize_sub_tree(
 
             if (status != AXIS2_SUCCESS)
             {
-                return status;
+                break;
             }
         }
 
@@ -824,6 +894,74 @@ axiom_node_serialize_sub_tree(
             {
                 if (om_node->data_element)
                 {
+
+                    axutil_hash_t *temp_attributes = NULL;
+                    axiom_namespace_t *namespace = NULL;
+                    /* at the writing of end part all the namespaces declared
+                       specially to that element should be cancelled */
+
+                    /* first checking the element namespace */
+                    namespace = axiom_element_get_namespace((axiom_element_t
+                                                             *) (om_node->
+                                                                 data_element), env,
+                                                            om_node);
+                    if (namespace)
+                    {
+                        axiom_namespace_t *ns = NULL;
+                        axis2_char_t *prefix = NULL;
+                        prefix = axiom_namespace_get_prefix(namespace, env);
+                        if (prefix)
+                        {
+                            ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                 AXIS2_HASH_KEY_STRING);
+                            if (ns)
+                            {
+                                axutil_hash_set(namespaces_from_parents, prefix,
+                                                    AXIS2_HASH_KEY_STRING, NULL);
+                            }
+                        }
+                    }
+                    
+                    /* then checking the attribute namespaces */
+
+                    temp_attributes = axiom_element_get_all_attributes((axiom_element_t *) (om_node->data_element), env);
+                    if (temp_attributes)
+                    {
+                        axutil_hash_index_t *hi;
+                        void *val;
+                        for (hi = axutil_hash_first(temp_attributes, env); hi;
+                             hi = axutil_hash_next(env, hi))
+                        {
+                            axutil_hash_this(hi, NULL, NULL, &val);
+
+                            if (val)
+                            {
+                                axiom_namespace_t *ns = NULL;
+                                axis2_char_t *prefix = NULL;
+
+                                namespace =
+                                    axiom_attribute_get_namespace((axiom_attribute_t
+                                                                   *) val, env);
+
+                                if (namespace)
+                                {
+                                    prefix =
+                                        axiom_namespace_get_prefix(namespace, env);
+                                    if (prefix)
+                                    {
+                                        ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                             AXIS2_HASH_KEY_STRING);
+                                        if (ns)
+                                        {
+                                            axutil_hash_set(namespaces_from_parents, prefix,
+                                                        AXIS2_HASH_KEY_STRING, NULL);
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+
                     status =
                         axiom_element_serialize_end_part((axiom_element_t
                                                           *) (om_node->
@@ -832,7 +970,7 @@ axiom_node_serialize_sub_tree(
                 }
                 if (status != AXIS2_SUCCESS)
                 {
-                    return status;
+                    break;
                 }
             }
 			/* We need to make make sure that om_node is not the root when we take the next sibling */
@@ -855,12 +993,79 @@ axiom_node_serialize_sub_tree(
                     {
                         if (om_node->data_element)
                         {
+                            axutil_hash_t *temp_attributes = NULL;
+                            axiom_namespace_t *namespace = NULL;
+
+                            /* similar to the earlier time, whenever the ending is happened
+                             * namespaces declared specially to that element should be cancelled */
+                         
+                            /* first checking the element namespace */
+                            namespace = axiom_element_get_namespace((axiom_element_t
+                                                                     *) (om_node->
+                                                                         data_element), env,
+                                                                    om_node);
+                            if (namespace)
+                            {
+                                axiom_namespace_t *ns = NULL;
+                                axis2_char_t *prefix = NULL;
+                                prefix = axiom_namespace_get_prefix(namespace, env);
+                                if (prefix)
+                                {
+                                    ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                         AXIS2_HASH_KEY_STRING);
+                                    if (ns)
+                                    {
+                                        axutil_hash_set(namespaces_from_parents, prefix,
+                                                            AXIS2_HASH_KEY_STRING, NULL);
+                                    }
+                                }
+                            }
+                            
+                            /* then checking the attribute namespaces */
+                         
+                            temp_attributes = axiom_element_get_all_attributes((axiom_element_t *) (om_node->data_element), env);
+                            if (temp_attributes)
+                            {
+                                axutil_hash_index_t *hi;
+                                void *val;
+                                for (hi = axutil_hash_first(temp_attributes, env); hi;
+                                     hi = axutil_hash_next(env, hi))
+                                {
+                                    axutil_hash_this(hi, NULL, NULL, &val);
+                         
+                                    if (val)
+                                    {
+                                        axiom_namespace_t *ns = NULL;
+                                        axis2_char_t *prefix = NULL;
+                         
+                                        namespace =
+                                            axiom_attribute_get_namespace((axiom_attribute_t
+                                                                           *) val, env);
+                         
+                                        if (namespace)
+                                        {
+                                            prefix =
+                                                axiom_namespace_get_prefix(namespace, env);
+                                            if (prefix)
+                                            {
+                                                ns = axutil_hash_get(namespaces_from_parents, prefix,
+                                                                     AXIS2_HASH_KEY_STRING);
+                                                if (ns)
+                                                {
+                                                    axutil_hash_set(namespaces_from_parents, prefix,
+                                                                AXIS2_HASH_KEY_STRING, NULL);
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
                             status =
                                 axiom_element_serialize_end_part((axiom_element_t *) (om_node->data_element), env, om_output);
                         }
                         if (status != AXIS2_SUCCESS)
                         {
-                            return status;
+                            break;
                         }
                     }
 
@@ -877,11 +1082,13 @@ axiom_node_serialize_sub_tree(
                     count--;
                 }
             }
-
         }
     }
     while (count > 0);
 
+    axutil_hash_free(namespaces_from_parents, env);
+    axutil_hash_free(namespaces, env);
+
     return status;
 }
 
