@@ -24,7 +24,7 @@
 #include <axiom_soap_const.h>
 #include <axutil_array_list.h>
 #include <axutil_uuid_gen.h>
-#include <axiom_mime_output.h>
+#include <axiom_mime_part.h>
 
 #define AXIS2_DEFAULT_CHAR_SET_ENCODING  "UTF-8"
 
@@ -60,12 +60,12 @@ struct axiom_output
 
     axutil_array_list_t *binary_node_list;
 
-    axiom_mime_output_t *mime_output;
-
     axis2_char_t *mime_boundry;
 
     axis2_char_t *content_type;
 
+    axutil_array_list_t *mime_parts;
+
 };
 
 AXIS2_EXTERN axiom_output_t *AXIS2_CALL
@@ -96,9 +96,9 @@ axiom_output_create(
     om_output->xml_version = NULL;
     om_output->ignore_xml_declaration = AXIS2_TRUE;
     om_output->binary_node_list = NULL;
-    om_output->mime_output = NULL;
     om_output->mime_boundry = NULL;
     om_output->content_type = NULL;
+    om_output->mime_parts = NULL;
 
     return om_output;
 }
@@ -137,10 +137,6 @@ axiom_output_free(
         axutil_array_list_free(om_output->binary_node_list, env);
     }
 
-    if (om_output->mime_output)
-    {
-        axiom_mime_output_free(om_output->mime_output, env);
-    }
     if (om_output->content_type)
     {
         AXIS2_FREE(env->allocator, om_output->content_type);
@@ -292,13 +288,9 @@ axiom_output_get_content_type(
 
         om_output->content_type =
             (axis2_char_t *)
-            axiom_mime_output_get_content_type_for_mime(om_output->mime_output,
-                                                        env,
-                                                        om_output->mime_boundry,
-                                                        om_output->
-                                                        root_content_id,
-                                                        om_output->
-                                                        char_set_encoding,
+            axiom_mime_part_get_content_type_for_mime(env, om_output->mime_boundry,
+                                                        om_output->root_content_id,
+                                                        om_output->char_set_encoding,
                                                         soap_content_type);
         return om_output->content_type;
     }
@@ -631,12 +623,16 @@ axiom_output_write_xml_version_encoding(
 
 }
 
-axis2_byte_t *AXIS2_CALL
-axiom_output_flush(
+/* This method will be called from transport. After this method each and every
+ * message part needs to be send are stored in an arraylits. So the transport 
+ * sender should correctly figure out how to send from the given information.
+ */ 
+
+
+AXIS2_EXTERN axis2_status_t AXIS2_CALL 
+    axiom_output_flush(
     axiom_output_t * om_output,
-    const axutil_env_t * env,
-    axis2_byte_t ** output_stream,
-    int *output_stream_size)
+    const axutil_env_t * env)
 {
     const axis2_char_t *soap_content_type = NULL;
 
@@ -644,12 +640,13 @@ axiom_output_flush(
 
     if (om_output->do_optimize)
     {
-        axis2_byte_t *byte_stream = NULL;
         axis2_char_t *root_content_id = NULL;
-        axis2_char_t *buffer =
-            (axis2_char_t *) axiom_xml_writer_get_xml(om_output->xml_writer,
+        axis2_char_t *buffer = NULL;
+
+        /* Extracting the soap part */
+
+        buffer = axiom_xml_writer_get_xml(om_output->xml_writer,
                                                       env);
-        int stream_size = 0;
         if (om_output->is_soap11)
         {
             soap_content_type = AXIOM_SOAP11_CONTENT_TYPE;
@@ -658,21 +655,41 @@ axiom_output_flush(
         {
             soap_content_type = AXIOM_SOAP12_CONTENT_TYPE;
         }
-        om_output->mime_output = axiom_mime_output_create(env);
+        
+        /* The created mime_boundary for this soap message */
+        
         om_output->mime_boundry = axiom_output_get_mime_boundry(om_output, env);
+        
+        /* This is also created for attachments*/
         root_content_id = axiom_output_get_root_content_id(om_output, env);
-        axiom_mime_output_complete(om_output->mime_output,
-                                   env, &byte_stream, &stream_size,
-                                   buffer, om_output->binary_node_list,
-                                   om_output->mime_boundry,
-                                   om_output->root_content_id,
-                                   om_output->char_set_encoding,
-                                   soap_content_type);
-
-        *output_stream = byte_stream;
-        *output_stream_size = stream_size;
-
-        return byte_stream;
+        
+        /* different parts of the message is added according to their order
+         * to an arraylist */
+        om_output->mime_parts = axiom_mime_part_create_part_list( 
+                                    env, buffer, om_output->binary_node_list,
+                                    om_output->mime_boundry, 
+                                    om_output->root_content_id, 
+                                    om_output->char_set_encoding,
+                                    soap_content_type);
+        
+        if(om_output->mime_parts)
+        {
+            return AXIS2_SUCCESS;
+        }
+        else
+        {
+            return AXIS2_FAILURE;
+        }
     }
-    return NULL;
+    return AXIS2_SUCCESS;
 }
+
+AXIS2_EXTERN axutil_array_list_t *AXIS2_CALL
+axiom_output_get_mime_parts(
+    axiom_output_t * om_output,
+    const axutil_env_t * env)
+{
+    AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
+    return om_output->mime_parts;
+}
+
