@@ -812,6 +812,10 @@ extern "C"
     axis2_conf_get_base(
         const axis2_conf_t * conf,
         const axutil_env_t * env);
+
+	AXIS2_EXTERN axutil_array_list_t * AXIS2_CALL
+	axis2_conf_get_handlers(const axis2_conf_t * conf,
+		const axutil_env_t * env);
 #ifdef __cplusplus
 }
 #endif
