@@ -1453,7 +1453,7 @@ extern "C"
      * @param supported_rest_http_methods pointer array list containing
      * the list of HTTP Methods supported. Message context does
      * assumes the ownership of the array list. Anything added to this
-     * arrary list will be freed by the msg_ctx
+     * array list will be freed by the msg_ctx
      * @return AXIS2_SUCCESS on success, else AXIS2_FAILURE
      */
     AXIS2_EXTERN axis2_status_t AXIS2_CALL
@@ -2060,7 +2060,36 @@ extern "C"
         const axutil_env_t * env,
         axutil_array_list_t * output_headers);
 
-    /** @} */
+
+
+    AXIS2_EXTERN axutil_array_list_t *AXIS2_CALL
+    axis2_msg_ctx_get_mime_parts(
+        axis2_msg_ctx_t * msg_ctx,
+        const axutil_env_t * env);
+
+
+    AXIS2_EXTERN void AXIS2_CALL
+    axis2_msg_ctx_set_mime_parts(
+        axis2_msg_ctx_t * msg_ctx,
+        const axutil_env_t * env,
+        axutil_array_list_t *mime_parts);
+
+    /**
+     * Incrementing the msg_ctx ref count. This is necessary when 
+     * prevent freeing msg_ctx through op_client when it is in use 
+     * as in sandesha2.
+     * @param msg_ctx pointer to message context  
+     * @param env pointer to environment struct
+     * @return AXIS2_TRUE if still in use, else AXIS2_FALSE 
+     */
+    AXIS2_EXTERN axis2_status_t AXIS2_CALL
+    axis2_msg_ctx_increment_ref(
+        axis2_msg_ctx_t * msg_ctx,
+        const axutil_env_t * env);
+
+
+
+/** @} */
 
 #ifdef __cplusplus
 }
