@@ -1,10 +1,10 @@
-
 /*
- * Copyright 2004,2005 The Apache Software Foundation.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -34,11 +34,20 @@ extern "C"
 {
 #endif
 
+    /**
+     * Builts EncryptedParts or SignedParts assertion
+     * @param env Pointer to environment struct
+     * @param node Assertion node
+     * @param element Assertion element
+     * @param is_signed boolean showing whether signing or encryption
+     * @returns neethi assertion created. NULL if failure.
+     */
     AXIS2_EXTERN neethi_assertion_t *AXIS2_CALL
     rp_signed_encrypted_parts_builder_build(
         const axutil_env_t * env,
         axiom_node_t * node,
-        axiom_element_t * element);
+        axiom_element_t * element, 
+        axis2_bool_t is_signed);
 
 #ifdef __cplusplus
 }
