@@ -17,64 +17,70 @@
 
 #include <neethi_engine.h>
 #include <neethi_assertion_builder.h>
+#include <axiom_attribute.h>
 
 /*Private functions*/
 
-neethi_all_t *AXIS2_CALL get_operator_all(
+static neethi_all_t *neethi_engine_get_operator_all(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element);
 
-neethi_exactlyone_t *AXIS2_CALL get_operator_exactlyone(
+static neethi_exactlyone_t *neethi_engine_get_operator_exactlyone(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element);
 
-neethi_reference_t *AXIS2_CALL get_operator_reference(
+static neethi_reference_t *neethi_engine_get_operator_reference(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element);
 
-neethi_policy_t *AXIS2_CALL get_operator_neethi_policy(
+static neethi_policy_t *neethi_engine_get_operator_neethi_policy(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element);
 
-axis2_status_t AXIS2_CALL process_operation_element(
+static axis2_status_t neethi_engine_process_operation_element(
     const axutil_env_t *env,
     neethi_operator_t *neethi_operator,
     axiom_node_t *node,
     axiom_element_t *element);
 
-axis2_status_t AXIS2_CALL neethi_engine_add_policy_component(
+static axis2_status_t neethi_engine_add_policy_component(
     const axutil_env_t *env,
     neethi_operator_t *container_operator,
     neethi_operator_t *component);
 
-axis2_bool_t AXIS2_CALL operator_is_empty(
+static axis2_bool_t neethi_engine_operator_is_empty(
     neethi_operator_t *operator,
     const axutil_env_t *env);
 
-neethi_exactlyone_t *AXIS2_CALL compute_resultant_component(
+static neethi_exactlyone_t *neethi_engine_compute_resultant_component(
     axutil_array_list_t *normalized_inner_components,
     neethi_operator_type_t type,
     const axutil_env_t *env);
 
-axutil_array_list_t *AXIS2_CALL operator_get_components(
+static axutil_array_list_t *neethi_engine_operator_get_components(
     neethi_operator_t *operator,
     const axutil_env_t *env);
 
-neethi_exactlyone_t *AXIS2_CALL normalize_operator(
+static neethi_exactlyone_t *neethi_engine_normalize_operator(
     neethi_operator_t *operator,
     neethi_registry_t *registry,
     axis2_bool_t deep,
     const axutil_env_t *env);
 
-neethi_exactlyone_t *AXIS2_CALL get_cross_product(
+static neethi_exactlyone_t *neethi_engine_get_cross_product(
     neethi_exactlyone_t *exactlyone1,
     neethi_exactlyone_t *exactlyone2,
     const axutil_env_t *env);
 
+static void neethi_engine_clear_element_attributes(
+    axutil_hash_t *attr_hash,
+    const axutil_env_t *env);
+
+
 /*Implementations*/
 
 /*This is the function which is called from outside*/
@@ -84,12 +90,13 @@ neethi_engine_get_policy(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element)
-    {
-        return get_operator_neethi_policy(env, node, element);
-    }
+{
+    /* This function will be called recursively */
 
-neethi_all_t *AXIS2_CALL
-get_operator_all(
+    return neethi_engine_get_operator_neethi_policy(env, node, element);
+}
+
+static neethi_all_t *neethi_engine_get_operator_all(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element)
@@ -114,7 +121,7 @@ get_operator_all(
     }
     neethi_operator_set_value(neethi_operator, env, all, OPERATOR_TYPE_ALL);
 
-    status = process_operation_element(env, neethi_operator, node, element);
+    status = neethi_engine_process_operation_element(env, neethi_operator, node, element);
 
     neethi_operator_set_value_null(neethi_operator, env);
     neethi_operator_free(neethi_operator, env);
@@ -132,8 +139,7 @@ get_operator_all(
     return all;
 }
 
-neethi_exactlyone_t *AXIS2_CALL
-get_operator_exactlyone(
+static neethi_exactlyone_t *neethi_engine_get_operator_exactlyone(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element)
@@ -161,7 +167,7 @@ get_operator_exactlyone(
     }
     neethi_operator_set_value(neethi_operator, env, exactlyone,
                               OPERATOR_TYPE_EXACTLYONE);
-    status = process_operation_element(env, neethi_operator, node, element);
+    status = neethi_engine_process_operation_element(env, neethi_operator, node, element);
 
     neethi_operator_set_value_null(neethi_operator, env);
     neethi_operator_free(neethi_operator, env);
@@ -182,8 +188,7 @@ get_operator_exactlyone(
     return exactlyone;
 }
 
-neethi_reference_t *AXIS2_CALL
-get_operator_reference(
+neethi_reference_t *neethi_engine_get_operator_reference(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element)
@@ -216,8 +221,11 @@ get_operator_reference(
     return reference;
 }
 
-neethi_policy_t *AXIS2_CALL
-get_operator_neethi_policy(
+/* This function will be called when we encounter a wsp:Policy
+ * element */
+
+
+static neethi_policy_t *neethi_engine_get_operator_neethi_policy(
     const axutil_env_t *env,
     axiom_node_t *node,
     axiom_element_t *element)
@@ -226,6 +234,8 @@ get_operator_neethi_policy(
     neethi_operator_t *neethi_operator = NULL;
     axis2_status_t status = AXIS2_SUCCESS;
 
+    /* Creates a policy struct */
+
     neethi_policy = neethi_policy_create(env);
 
     if (!neethi_policy)
@@ -234,6 +244,9 @@ get_operator_neethi_policy(
         AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Out of memory");
         return NULL;
     }
+
+    /* Then we wrap it in a neethi_operator */
+
     neethi_operator = neethi_operator_create(env);
     if (!neethi_operator)
     {
@@ -243,7 +256,15 @@ get_operator_neethi_policy(
     }
     neethi_operator_set_value(neethi_operator, env, neethi_policy,
                               OPERATOR_TYPE_POLICY);
-    status = process_operation_element(env, neethi_operator, node, element);
+
+    /* This function will do all the processing and build the 
+     * policy object model */
+
+    status = neethi_engine_process_operation_element(env, neethi_operator, node, element);
+
+    /* To prevent freeing the policy object from the operator
+     * we set it to null. This object will be freed from a parent 
+     * or from an outsider who creates a policy object */
 
     neethi_operator_set_value_null(neethi_operator, env);
     neethi_operator_free(neethi_operator, env);
@@ -263,8 +284,12 @@ get_operator_neethi_policy(
     return neethi_policy;
 }
 
-axis2_status_t AXIS2_CALL
-process_operation_element(
+/* This function will construct the policy objecy model by 
+ * filling the component array_list inside the passing 
+ * policy operator */
+
+
+static axis2_status_t neethi_engine_process_operation_element(
     const axutil_env_t *env,
     neethi_operator_t *neethi_operator,
     axiom_node_t *node,
@@ -279,47 +304,80 @@ process_operation_element(
 
     type = neethi_operator_get_type(neethi_operator, env);
     value = neethi_operator_get_value(neethi_operator, env);
+
     if (type == OPERATOR_TYPE_POLICY)
     {
-        /*Adding attribute values to the hashmap logic
-         *comes here.Following is a little hack until we implement attribute
-         hash map logic.*/
+        /* wsp:Policy element can have any number of attributes
+         * we will store them in a hash from the uri and localname */
 
-        axis2_char_t *id = NULL;
-        axutil_qname_t *qname = NULL;
-        axis2_char_t *name = NULL;
+        axutil_hash_t *attributes = axiom_element_extract_attributes(
+            element, env, node);
 
-        qname =
-            axutil_qname_create(env, NEETHI_ID, NEETHI_WSU_NS,
-                                NULL/*NEETHI_WSU_NS_PREFIX*/);
-        if (!qname)
+        if(attributes)
         {
-            AXIS2_ERROR_SET(env->error, AXIS2_ERROR_NO_MEMORY, AXIS2_FAILURE);
-            AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Out of memory");
-            return AXIS2_FAILURE;
-        }
-        id = axiom_element_get_attribute_value(element, env, qname);
-        axutil_qname_free(qname, env);
-        qname = NULL;
+            axutil_hash_index_t *hi = NULL;            
+        
+            /* When creating the policy object we created the hash */        
+    
+            axutil_hash_t *ht = neethi_policy_get_attributes(
+                (neethi_policy_t *)value, env);
 
-        qname = axutil_qname_create(env, NEETHI_NAME, NULL, NULL);
-        if (!qname)
-        {
-            AXIS2_ERROR_SET(env->error, AXIS2_ERROR_NO_MEMORY, AXIS2_FAILURE);
-            AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Out of memory");
-            return AXIS2_FAILURE;
-        }
-        name = axiom_element_get_attribute_value(element, env, qname);
-        axutil_qname_free(qname, env);
-        qname = NULL;
+            if(!ht)
+            {
+                AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                    "[neethi] Policy hash map creation failed.");
+                return AXIS2_FAILURE;
+            }
+            
 
-        if (id)
-        {
-            neethi_policy_set_id((neethi_policy_t *) value, env, id);
-        }
-        if (name)
-        {
-            neethi_policy_set_name((neethi_policy_t *) value, env, name);
+            for (hi = axutil_hash_first(attributes, env);
+                hi; hi = axutil_hash_next(env, hi))
+            {
+                axis2_char_t *key = NULL;
+                void *val = NULL;
+                axutil_qname_t *qname = NULL;
+                axis2_char_t *attr_val = NULL;    
+                axiom_namespace_t *ns = NULL;
+                axis2_char_t *ns_uri = NULL;
+                axiom_attribute_t *om_attr = NULL;                
+    
+                axutil_hash_this(hi, NULL, NULL, &val);
+                if(val)
+                {
+                    om_attr = (axiom_attribute_t *) val;
+                    ns = axiom_attribute_get_namespace(om_attr, env);
+                    if(ns)
+                    {
+                        ns_uri = axiom_namespace_get_uri(ns, env);
+                    }
+
+                    qname = axutil_qname_create(env, 
+                        axiom_attribute_get_localname(om_attr, env),
+                        ns_uri, NULL);
+                    if(qname)
+                    {
+                        key = axutil_qname_to_string(qname, env);
+                        if(key)
+                        {
+                            attr_val = axiom_attribute_get_value(om_attr, env);
+                            if(attr_val)
+                            {
+                                /* axutil_qname_free will free the returned key 
+                                 * of the qname so will duplicate it when storing */                                
+
+                                axutil_hash_set(ht, axutil_strdup(env,key), AXIS2_HASH_KEY_STRING, 
+                                    axutil_strdup(env, attr_val));
+                            }                                
+                        }   
+                        axutil_qname_free(qname, env);  
+                    }
+                }    
+            }
+            /* axiom_element_extract_attributes method will always returns 
+             * a cloned copy, so we need to free it after we have done with it */
+
+            neethi_engine_clear_element_attributes(attributes, env);
+            attributes = NULL;
         }
     }
 
@@ -328,6 +386,11 @@ process_operation_element(
     {
         while (axiom_children_iterator_has_next(children_iter, env))
         {
+            /* Extract the element and check the namespace. If the namespace 
+             * is in ws_policy then we call the relevent operator builder 
+             * otherwise we will call the assertion_builder */
+
+
             child_node = axiom_children_iterator_next(children_iter, env);
             if (child_node)
             {
@@ -367,13 +430,19 @@ process_operation_element(
                                             "[neethi] Invalid Empty namespace uri.");
                             return AXIS2_FAILURE;
                         }
-                        if (axutil_strcmp(uri, NEETHI_NAMESPACE) == 0)
+                        if ((axutil_strcmp(uri, NEETHI_NAMESPACE) == 0) || 
+                            (axutil_strcmp(uri, NEETHI_POLICY_15_NAMESPACE) == 0))
                         {
+                            /* Looking at the localname we will call the relevent 
+                             * operator function. After that the newly created 
+                             * object is wrapped in a neethi_operator and stored in 
+                             * the parent's component list */
+
                             if (axutil_strcmp(local_name, NEETHI_POLICY) == 0)
                             {
                                 neethi_policy_t *neethi_policy = NULL;
                                 neethi_policy =
-                                    get_operator_neethi_policy(env, child_node,
+                                    neethi_engine_get_operator_neethi_policy(env, child_node,
                                                                child_element);
                                 if (neethi_policy)
                                 {
@@ -400,7 +469,7 @@ process_operation_element(
                             {
                                 neethi_all_t *all = NULL;
                                 all =
-                                    get_operator_all(env, child_node,
+                                    neethi_engine_get_operator_all(env, child_node,
                                                      child_element);
                                 if (all)
                                 {
@@ -428,7 +497,7 @@ process_operation_element(
                             {
                                 neethi_exactlyone_t *exactlyone = NULL;
                                 exactlyone =
-                                    get_operator_exactlyone(env, child_node,
+                                    neethi_engine_get_operator_exactlyone(env, child_node,
                                                             child_element);
                                 if (exactlyone)
                                 {
@@ -456,18 +525,15 @@ process_operation_element(
                             {
                                 neethi_reference_t *reference = NULL;
                                 reference =
-                                    get_operator_reference(env, child_node,
+                                    neethi_engine_get_operator_reference(env, child_node,
                                                            child_element);
                                 if (reference)
                                 {
-                                    operator = neethi_operator_create(
-    env);
-                                    neethi_operator_set_value(operator, env,
-                                                              reference,
-                                                              OPERATOR_TYPE_REFERENCE);
-                                    neethi_engine_add_policy_component(env,
-                                                                       neethi_operator,
-                                                                       operator);
+                                    operator = neethi_operator_create(env);
+                                    neethi_operator_set_value(operator, env, reference, 
+                                        OPERATOR_TYPE_REFERENCE);
+                                    neethi_engine_add_policy_component(env, neethi_operator,
+                                        operator);
                                 }
                                 else
                                 {
@@ -480,22 +546,22 @@ process_operation_element(
                                 }
                             }
                         }
+
                         else
                         {
+                            /* This is an assertion in a different domain. Assertion builder
+                             * should be called and that will call the relevent assertion builder 
+                             * after looking at the localname and the namespace */
+                
                             neethi_assertion_t *assertion = NULL;
                             assertion =
-                                neethi_assertion_builder_build(env, child_node,
-                                                               child_element);
+                                neethi_assertion_builder_build(env, child_node, child_element);
                             if (assertion)
                             {
-                                operator = neethi_operator_create(
-    env);
-                                neethi_operator_set_value(operator, env,
-                                                          assertion,
+                                operator = neethi_operator_create(env);
+                                neethi_operator_set_value(operator, env, assertion,
                                                           OPERATOR_TYPE_ASSERTION);
-                                neethi_engine_add_policy_component(env,
-                                                                   neethi_operator,
-                                                                   operator);
+                                neethi_engine_add_policy_component(env, neethi_operator, operator);
                             }
                             else
                             {
@@ -517,8 +583,12 @@ process_operation_element(
         return AXIS2_FAILURE;
 }
 
-axis2_status_t AXIS2_CALL
-neethi_engine_add_policy_component(
+
+/* After looking at the operator_type this function will 
+ * call the relevent neethi operator's add operator 
+ * function */
+
+static axis2_status_t neethi_engine_add_policy_component(
     const axutil_env_t *env,
     neethi_operator_t *container_operator,
     neethi_operator_t *component)
@@ -607,6 +677,20 @@ check_neethi_policy(
 }
 
 /************************************************/
+/*
+
+Following function will normalize accorading to the 
+WS-Policy spec. Normalize policy is in the following 
+format.
+
+<wsp:Policy>
+    <wsp:ExactlyOne>
+        ( <wsp:All> ( <Assertion …> … </Assertion> )* </wsp:All> )*
+   </wsp:ExactlyOne>
+</wsp:Policy> 
+
+*/
+
 
 AXIS2_EXTERN neethi_policy_t *AXIS2_CALL
 neethi_engine_get_normalize(
@@ -614,9 +698,13 @@ neethi_engine_get_normalize(
     axis2_bool_t deep,
     neethi_policy_t *neethi_policy)
 {
+    /* In the first call we pass the registry as null.*/
+
     return neethi_engine_normalize(env, neethi_policy, NULL, deep);
 }
 
+
+
 AXIS2_EXTERN neethi_policy_t *AXIS2_CALL
 neethi_engine_normalize(
     const axutil_env_t *env,
@@ -631,6 +719,9 @@ neethi_engine_normalize(
     axis2_char_t *policy_name = NULL;
     axis2_char_t *policy_id = NULL;
 
+    /* Normalize policy will be contained in the new policy
+     * created below */
+
     resultant_neethi_policy = neethi_policy_create(env);
     if (!resultant_neethi_policy)
     {
@@ -650,8 +741,7 @@ neethi_engine_normalize(
         neethi_policy_set_id(resultant_neethi_policy, env, policy_id);
     }
 
-    operator = neethi_operator_create(
-    env);
+    operator = neethi_operator_create(env);
     if (!operator)
     {
         AXIS2_ERROR_SET(env->error, AXIS2_ERROR_NO_MEMORY, AXIS2_FAILURE);
@@ -659,15 +749,22 @@ neethi_engine_normalize(
         return NULL;
     }
 
-    neethi_operator_set_value(operator, env, neethi_policy,
-                              OPERATOR_TYPE_POLICY);
+    neethi_operator_set_value(operator, env, neethi_policy, OPERATOR_TYPE_POLICY);
 
-    exactlyone = normalize_operator(operator, registry, deep, env);
+    /* When we call the normalization it should always return an exactlyone as the 
+     * out put. */
+
+    exactlyone = neethi_engine_normalize_operator(operator, registry, deep, env);
+
+    /* We are frreing the operator used to wrap the object */
 
     neethi_operator_set_value_null(operator, env);
     neethi_operator_free(operator, env);
     operator = NULL;
 
+    /* This exactlyone is set as the first component of the 
+     * normalized policy */
+
     if (exactlyone)
     {
         component = neethi_operator_create(env);
@@ -711,7 +808,7 @@ neethi_engine_merge(
                         "[neethi] Wrong input for merge.");
         return NULL;
     }
-    exactlyone = get_cross_product(exactlyone1, exactlyone2, env);
+    exactlyone = neethi_engine_get_cross_product(exactlyone1, exactlyone2, env);
     if (exactlyone)
     {
         neethi_policy = neethi_policy_create(env);
@@ -737,8 +834,7 @@ neethi_engine_merge(
     }
 }
 
-axis2_bool_t AXIS2_CALL
-operator_is_empty(
+static axis2_bool_t neethi_engine_operator_is_empty(
     neethi_operator_t *operator,
     const axutil_env_t *env)
 {
@@ -791,8 +887,7 @@ operator_is_empty(
         return AXIS2_FALSE;
 }
 
-axutil_array_list_t *AXIS2_CALL
-operator_get_components(
+static axutil_array_list_t *neethi_engine_operator_get_components(
     neethi_operator_t *operator,
     const axutil_env_t *env)
 {
@@ -843,8 +938,7 @@ operator_get_components(
     return NULL;
 }
 
-neethi_exactlyone_t *AXIS2_CALL
-normalize_operator(
+static neethi_exactlyone_t *neethi_engine_normalize_operator(
     neethi_operator_t *operator,
     neethi_registry_t *registry,
     axis2_bool_t deep,
@@ -857,8 +951,11 @@ normalize_operator(
 
     neethi_operator_type_t type = neethi_operator_get_type(operator, env);
 
-    if (operator_is_empty(operator, env))
+    if (neethi_engine_operator_is_empty(operator, env))
     {
+        /* If this is an empty operator we just add 
+         * an exactlyone and all */
+
         neethi_exactlyone_t *exactlyone = NULL;
         exactlyone = neethi_exactlyone_create(env);
         if (!exactlyone)
@@ -887,7 +984,9 @@ normalize_operator(
     }
 
     child_component_list = axutil_array_list_create(env, 0);
-    arraylist = operator_get_components(operator, env);
+    arraylist = neethi_engine_operator_get_components(operator, env);
+
+    /* Here we are recursively normalize each and every component */
 
     for (i = 0; i < axutil_array_list_size(arraylist, env); i++)
     {
@@ -920,6 +1019,9 @@ normalize_operator(
                     AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Out of memory");
                     return NULL;
                 }
+
+                /* We wrap everything inside an exactlyone */
+
                 neethi_all_add_operator(all, env, child_component);
                 neethi_operator_set_value(op, env, all, OPERATOR_TYPE_ALL);
                 neethi_exactlyone_add_operator(exactlyone, env, op);
@@ -963,7 +1065,7 @@ normalize_operator(
                     neethi_operator_set_value(to_normalize, env, all,
                                               OPERATOR_TYPE_ALL);
                     exactlyone =
-                        normalize_operator(to_normalize, registry, deep, env);
+                        neethi_engine_normalize_operator(to_normalize, registry, deep, env);
                     if (exactlyone)
                     {
                         axutil_array_list_add(child_component_list, env,
@@ -984,6 +1086,12 @@ normalize_operator(
 
         else if (component_type == OPERATOR_TYPE_REFERENCE)
         {
+
+            /* If the operator is a policy reference we will 
+             * extract the relevent policy from the uri and 
+             * normalize as we are doing for a neethi_policy 
+             * object */
+
             neethi_reference_t *policy_ref = NULL;
             axis2_char_t *uri = NULL;
             neethi_policy_t *policy = NULL;
@@ -1005,6 +1113,14 @@ normalize_operator(
                                 "[neethi] Uri not specified");
                 return NULL;
             }
+
+            if(!registry)
+            {
+                AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                                "[neethi] Cannot resolve the reference.Registry Not provided");
+                return NULL;
+            }
+
             policy = neethi_registry_lookup(registry, env, uri);
             if (!policy)
             {
@@ -1047,7 +1163,7 @@ normalize_operator(
                     neethi_operator_set_value(to_normalize, env, all,
                                               OPERATOR_TYPE_ALL);
                     exactlyone =
-                        normalize_operator(to_normalize, registry, deep, env);
+                        neethi_engine_normalize_operator(to_normalize, registry, deep, env);
                     if (exactlyone)
                     {
                         axutil_array_list_add(child_component_list, env,
@@ -1069,7 +1185,7 @@ normalize_operator(
         {
             neethi_exactlyone_t *exactlyone = NULL;
             exactlyone =
-                normalize_operator(child_component, registry, deep, env);
+                neethi_engine_normalize_operator(child_component, registry, deep, env);
             if (exactlyone)
             {
                 axutil_array_list_add(child_component_list, env, exactlyone);
@@ -1077,25 +1193,39 @@ normalize_operator(
         }
     }
 
-    return compute_resultant_component(child_component_list, type, env);
+    /* So at this point we have set of exactlyones in the array_list, So we will 
+     * compute one exactlyone out of those after the following call */
+
+    return neethi_engine_compute_resultant_component(child_component_list, type, env);
 }
 
-neethi_exactlyone_t *AXIS2_CALL
-compute_resultant_component(
+/* This function will return a single exactlyone from all the 
+ * components in the list */
+
+
+static neethi_exactlyone_t *neethi_engine_compute_resultant_component(
     axutil_array_list_t * normalized_inner_components,
     neethi_operator_type_t type,
     const axutil_env_t * env)
 {
     neethi_exactlyone_t *exactlyone = NULL;
+    int size = 0;
+
+    if(normalized_inner_components)
+    {
+        size = axutil_array_list_size(normalized_inner_components, env);
+    }
 
     if (type == OPERATOR_TYPE_EXACTLYONE)
     {
+        /* If the operator is an exactlyone then we get all the components
+         * in the exatlyones and add them to a newly created exactlyone */
+
         int i = 0;
         neethi_exactlyone_t *inner_exactlyone = NULL;
         exactlyone = neethi_exactlyone_create(env);
 
-        for (i = 0;
-             i < axutil_array_list_size(normalized_inner_components, env); i++)
+        for (i = 0; i < size; i++)
         {
             inner_exactlyone =
                 (neethi_exactlyone_t *)
@@ -1120,12 +1250,18 @@ compute_resultant_component(
     }
     else if (type == OPERATOR_TYPE_POLICY || type == OPERATOR_TYPE_ALL)
     {
-        if (axutil_array_list_size(normalized_inner_components, env) > 1)
+        /* Here arry_list contains one exactlyone means this operator 
+         * is already normalized. So we will return that. Otherwise we 
+         * will get the crossproduct. */
+
+        if (size > 1)
         {
+            /* Get the first one and do the cross product with other 
+             * components */
+
             int i = 0;
-            exactlyone =
-                (neethi_exactlyone_t *)
-                axutil_array_list_get(normalized_inner_components, env, 0);
+            exactlyone = (neethi_exactlyone_t *)axutil_array_list_get
+                (normalized_inner_components, env, 0);
             if (!exactlyone)
             {
                 AXIS2_ERROR_SET(env->error,
@@ -1139,14 +1275,10 @@ compute_resultant_component(
             {
                 neethi_exactlyone_t *current_exactlyone = NULL;
                 i = 1;
-                for (i = 1;
-                     i < axutil_array_list_size(normalized_inner_components,
-                                                env); i++)
+                for (i = 1; i < size; i++)
                 {
-                    current_exactlyone =
-                        (neethi_exactlyone_t *)
-                        axutil_array_list_get(normalized_inner_components, env,
-                                              i);
+                    current_exactlyone = (neethi_exactlyone_t *)
+                        axutil_array_list_get(normalized_inner_components, env, i);
                     if (!current_exactlyone)
                     {
                         AXIS2_ERROR_SET(env->error,
@@ -1167,9 +1299,8 @@ compute_resultant_component(
                         neethi_exactlyone_t *temp1 = NULL;
                         temp = exactlyone;
                         temp1 = current_exactlyone;
-                        exactlyone =
-                            get_cross_product(exactlyone, current_exactlyone,
-                                              env);
+                        exactlyone = neethi_engine_get_cross_product(
+                            exactlyone, current_exactlyone, env);
                         neethi_exactlyone_free(temp, env);
                         neethi_exactlyone_free(temp1, env);
                         temp = NULL;
@@ -1189,9 +1320,8 @@ compute_resultant_component(
         }
         else
         {
-            exactlyone =
-                (neethi_exactlyone_t *)
-                axutil_array_list_get(normalized_inner_components, env, 0);
+            exactlyone = (neethi_exactlyone_t *)axutil_array_list_get(
+                normalized_inner_components, env, 0);
         }
     }
     axutil_array_list_free(normalized_inner_components, env);
@@ -1200,8 +1330,11 @@ compute_resultant_component(
     return exactlyone;
 }
 
-neethi_exactlyone_t *AXIS2_CALL
-get_cross_product(
+/* The cross product will return all the different combinations 
+ * of alternatives and put them into one exactlyone */
+
+
+static neethi_exactlyone_t *neethi_engine_get_cross_product(
     neethi_exactlyone_t *exactlyone1,
     neethi_exactlyone_t *exactlyone2,
     const axutil_env_t *env)
@@ -1238,7 +1371,8 @@ get_cross_product(
 
     for (i = 0; i < axutil_array_list_size(array_list1, env); i++)
     {
-        current_all1 = (neethi_all_t *) neethi_operator_get_value((neethi_operator_t *) axutil_array_list_get(array_list1, env, i), env);
+        current_all1 = (neethi_all_t *) neethi_operator_get_value(
+            (neethi_operator_t *) axutil_array_list_get(array_list1, env, i), env);
 
         if (!current_all1)
         {
@@ -1252,7 +1386,8 @@ get_cross_product(
 
         for (j = 0; j < axutil_array_list_size(array_list2, env); j++)
         {
-            current_all2 = (neethi_all_t *) neethi_operator_get_value((neethi_operator_t *) axutil_array_list_get(array_list2, env, j), env);
+            current_all2 = (neethi_all_t *) neethi_operator_get_value(
+                (neethi_operator_t *) axutil_array_list_get(array_list2, env, j), env);
 
             if (!current_all2)
             {
@@ -1306,3 +1441,25 @@ neethi_engine_serialize(
 
     return neethi_policy_serialize(policy, NULL, env);
 }
+
+static void neethi_engine_clear_element_attributes(
+    axutil_hash_t *attr_hash,
+    const axutil_env_t *env)
+{
+    axutil_hash_index_t *hi = NULL;
+
+    for(hi = axutil_hash_first(attr_hash, env); hi; hi = axutil_hash_next(env, hi))
+    {
+        void *val = NULL;
+        axutil_hash_this(hi, NULL, NULL, &val);
+        if (val)
+        {
+            axiom_attribute_free((axiom_attribute_t *)val, env);
+            val = NULL;
+        }
+    }
+    axutil_hash_free(attr_hash, env);
+    attr_hash = NULL;
+
+    return;
+}
