@@ -82,9 +82,9 @@ axis2_options_create(
     options->use_separate_listener = -1;
     options->receiver = NULL;
     options->transport_in = NULL;
-    options->transport_in_protocol = AXIS2_TRANSPORT_ENUM_HTTP;
+    options->transport_in_protocol = AXIS2_TRANSPORT_ENUM_MAX;
     options->transport_out = NULL;
-    options->sender_transport_protocol = AXIS2_TRANSPORT_ENUM_HTTP;
+    options->sender_transport_protocol = AXIS2_TRANSPORT_ENUM_MAX;
     options->manage_session = -1;
     options->soap_version = AXIOM_SOAP12;
     options->enable_mtom = AXIS2_FALSE;
@@ -659,7 +659,23 @@ axis2_options_set_use_separate_listener(
     const axutil_env_t * env,
     const axis2_bool_t use_separate_listener)
 {
+    axutil_property_t *property = NULL;
+
     options->use_separate_listener = use_separate_listener;
+
+    if (use_separate_listener)
+    {
+        property = axutil_property_create(env);
+        axutil_property_set_value(property, env, axutil_strdup(env, AXIS2_VALUE_TRUE));
+        axis2_options_set_property(options, env, AXIS2_USE_SEPARATE_LISTENER, property);
+    }
+    else
+    {
+        property = axutil_property_create(env);
+        axutil_property_set_value(property, env, axutil_strdup(env, AXIS2_VALUE_FALSE));
+        axis2_options_set_property(options, env, AXIS2_USE_SEPARATE_LISTENER, property);
+    }
+    
     return AXIS2_SUCCESS;
 }
 
