@@ -78,6 +78,8 @@ struct axis2_svc_client
     axutil_array_list_t *http_headers;
 
     int http_status_code;
+    
+    axis2_bool_t keep_externally_passed_ctx_and_svc;
 
 };
 
@@ -121,128 +123,6 @@ axis2_svc_client_create(
     return svc_client;
 }
 
-AXIS2_EXTERN axis2_svc_client_t *AXIS2_CALL
-axis2_svc_client_create_for_dynamic_invocation(
-    const axutil_env_t * env,
-    axis2_conf_ctx_t * conf_ctx,
-    const axutil_uri_t * wsdl_uri,
-    const axutil_qname_t * wsdl_svc_qname,
-    const axis2_char_t * endpoint_name,
-    const axis2_char_t * client_home)
-{
-    axis2_svc_client_t *svc_client = NULL;
-    axis2_svc_grp_t *svc_grp = NULL;
-    axis2_svc_grp_ctx_t *svc_grp_ctx = NULL;
-    const axis2_char_t *svc_grp_name = NULL;
-    axutil_hash_t *ops = NULL;
-    const axis2_char_t *repos_path = NULL;
-    axis2_char_t *wsdl_path = NULL;
-
-    AXIS2_ENV_CHECK(env, NULL);
-
-    svc_client = AXIS2_MALLOC(env->allocator, sizeof(axis2_svc_client_t));
-    if (!svc_client)
-    {
-        AXIS2_ERROR_SET(env->error, AXIS2_ERROR_NO_MEMORY, AXIS2_FAILURE);
-        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "No memory. Cannot create service client.");
-        return NULL;
-    }
-
-    svc_client->svc = NULL;
-    svc_client->conf = NULL;
-    svc_client->conf_ctx = NULL;
-    svc_client->svc_ctx = NULL;
-    svc_client->options = NULL;
-    svc_client->override_options = NULL;
-    svc_client->headers = NULL;
-    svc_client->callback_recv = NULL;
-    svc_client->listener_manager = NULL;
-    svc_client->op_client = NULL;
-    svc_client->http_headers = NULL;
-    svc_client->http_status_code = 0;
-
-    if (!axis2_svc_client_init_data(env, svc_client))
-    {
-        axis2_svc_client_free(svc_client, env);
-        return NULL;
-    }
-
-    /*create the default conf_ctx if it is NULL */
-    if (!axis2_svc_client_init_transports_from_conf_ctx(env, svc_client,
-                                                        conf_ctx, client_home))
-    {
-        axis2_svc_client_free(svc_client, env);
-        return NULL;
-    }
-
-    svc_client->conf = axis2_conf_ctx_get_conf(svc_client->conf_ctx, env);
-    repos_path = axis2_conf_get_repo(svc_client->conf, env);
-    wsdl_path =
-        axutil_strcat(env, repos_path, AXIS2_PATH_SEP_STR, AXIS2_WSDL_LOCATION_IN_REPO, NULL);
-
-    svc_client->options = axis2_options_create(env);
-    if (svc_client->svc)
-    {
-        axutil_hash_index_t *i = NULL;
-        void *v = NULL;
-        axis2_op_t *op = NULL;
-
-        ops = axis2_svc_get_all_ops(svc_client->svc, env);
-        for (i = axutil_hash_first(ops, env); i; i = axutil_hash_next(env, i))
-        {
-            axis2_phases_info_t *info = NULL;
-            axutil_hash_this(i, NULL, NULL, &v);
-            op = (axis2_op_t *) v;
-
-            /* Setting operation phase */
-            info = axis2_conf_get_phases_info(svc_client->conf, env);
-            axis2_phases_info_set_op_phases(info, env, op);
-        }
-    }
-    else
-        return NULL;
-
-    /** add the service to the config context if it isn't in there already */
-    if (!axis2_conf_get_svc(svc_client->conf, env,
-                            axis2_svc_get_name(svc_client->svc, env)))
-    {
-        axis2_conf_add_svc(svc_client->conf, env, svc_client->svc);
-    }
-
-    /** create a service context for myself: create a new service group
-     context and then get the service context for myself as I'll need that
-     later for stuff that I gotta do */
-    svc_grp = axis2_svc_get_parent(svc_client->svc, env);
-    if (!svc_grp)
-    {
-        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Cannot access service group of service client.");
-        return NULL;
-    }
-
-    svc_grp_ctx = axis2_svc_grp_get_svc_grp_ctx(svc_grp, env,
-                                                svc_client->conf_ctx);
-    if (!svc_grp_ctx)
-    {
-        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Cannot access service group context of service client.");
-        return NULL;
-    }
-
-    svc_grp_name = axis2_svc_grp_get_name(svc_grp, env);
-    if (!svc_grp_name)
-    {
-        AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Cannot access service group name of service client.");
-        return NULL;            /* service group name is mandatory */
-    }
-
-    axis2_conf_ctx_register_svc_grp_ctx(svc_client->conf_ctx, env,
-                                        svc_grp_name, svc_grp_ctx);
-
-    svc_client->svc_ctx = axis2_svc_grp_ctx_get_svc_ctx(svc_grp_ctx, env,
-                                                        axis2_svc_get_name
-                                                        (svc_client->svc, env));
-
-    return svc_client;
-}
 
 AXIS2_EXTERN axis2_svc_client_t *AXIS2_CALL
 axis2_svc_client_create_with_conf_ctx_and_svc(
@@ -281,6 +161,7 @@ axis2_svc_client_create_with_conf_ctx_and_svc(
     svc_client->required_auth_is_http = AXIS2_FALSE;
     svc_client->auth_type = NULL;
 	svc_client->http_headers = NULL;
+    svc_client->keep_externally_passed_ctx_and_svc = AXIS2_FALSE;
 
     if (!axis2_svc_client_init_data(env, svc_client))
     {
@@ -300,6 +181,7 @@ axis2_svc_client_create_with_conf_ctx_and_svc(
 
     if (svc)
     {
+        svc_client->keep_externally_passed_ctx_and_svc = AXIS2_TRUE;
         svc_client->svc = svc;
     }
     else
@@ -364,6 +246,15 @@ axis2_svc_client_get_svc(
     return svc_client->svc;
 }
 
+AXIS2_EXTERN axis2_conf_ctx_t *AXIS2_CALL
+axis2_svc_client_get_conf_ctx(
+    const axis2_svc_client_t * svc_client,
+    const axutil_env_t * env)
+{
+    AXIS2_PARAM_CHECK (env->error, svc_client, NULL);
+    return svc_client->conf_ctx;
+}
+
 AXIS2_EXTERN axis2_status_t AXIS2_CALL
 axis2_svc_client_set_options(
     axis2_svc_client_t * svc_client,
@@ -663,7 +554,7 @@ axis2_svc_client_fire_and_forget_with_op_qname(
     }
 
     axis2_op_client_add_out_msg_ctx(svc_client->op_client, env, msg_ctx);
-    axis2_op_client_execute(svc_client->op_client, env, AXIS2_FALSE);
+    axis2_op_client_execute(svc_client->op_client, env, AXIS2_TRUE);
     axis2_svc_client_set_http_info(svc_client, env, msg_ctx);
     svc_client->auth_failed = axis2_msg_ctx_get_auth_failed(msg_ctx, env);
     svc_client->required_auth_is_http =
@@ -746,7 +637,7 @@ axis2_svc_client_send_receive_with_op_qname(
 
     /* If dual channel blocking. We come to this block if the client indicate to use
      * a separate listener but don't provide a callback function to acted upon when
-     * response is recieved in the listener thread. What we do here is we create a callback
+     * response is received in the listener thread. What we do here is we create a callback
      * and call axis2_svc_client_send_receive_non_blocking_with_op_qname with it. */
     if (axis2_options_get_use_separate_listener(svc_client->options, env))
     {
@@ -994,9 +885,14 @@ axis2_svc_client_send_receive_non_blocking_with_op_qname(
     if (axis2_options_get_use_separate_listener(svc_client->options, env))
     {
         axis2_op_t *op = NULL;
-
+		
         transport_in_protocol =
             axis2_options_get_transport_in_protocol(svc_client->options, env);
+		if (transport_in_protocol == AXIS2_TRANSPORT_ENUM_MAX)
+		{
+			axis2_options_set_transport_in_protocol(svc_client->options, env, AXIS2_TRANSPORT_ENUM_HTTP);
+			transport_in_protocol = AXIS2_TRANSPORT_ENUM_HTTP;
+		}
         axis2_listener_manager_make_sure_started(svc_client->listener_manager,
                                                  env, transport_in_protocol,
                                                  svc_client->conf_ctx);
@@ -1160,6 +1056,10 @@ axis2_svc_client_init_transports_from_conf_ctx(
             return AXIS2_FALSE;
         }
     }
+    else
+    {
+        svc_client->keep_externally_passed_ctx_and_svc = AXIS2_TRUE;
+    }
 
     if (!svc_client->listener_manager)
     {
@@ -1315,7 +1215,7 @@ axis2_svc_client_free(
         svc_client->headers = NULL;        
     }
 
-    if (svc_client->svc)
+    if (svc_client->svc && !svc_client->keep_externally_passed_ctx_and_svc)
     {
         axis2_svc_free(svc_client->svc, env);
     }
@@ -1328,6 +1228,7 @@ axis2_svc_client_free(
     if (svc_client->op_client)
     {
         axis2_op_client_free(svc_client->op_client, env);
+        svc_client->op_client = NULL;
     }
 
     if (svc_client->options)
@@ -1340,7 +1241,7 @@ axis2_svc_client_free(
         axis2_listener_manager_free(svc_client->listener_manager, env);
     }
 
-    if (svc_client->conf_ctx)
+    if (svc_client->conf_ctx && !svc_client->keep_externally_passed_ctx_and_svc)
     {
         axis2_conf_ctx_free(svc_client->conf_ctx, env);
     }
@@ -1568,7 +1469,9 @@ axis2_svc_client_set_proxy_with_auth(
         host_attr = axiom_attribute_create(env, AXIS2_HTTP_PROXY_HOST, proxy_host, NULL);
         port_attr = axiom_attribute_create(env, AXIS2_HTTP_PROXY_PORT, proxy_port, NULL);
         axutil_generic_obj_set_value(host_obj, env, host_attr);
+        axutil_generic_obj_set_free_func(host_obj, env, axiom_attribute_free_void_arg);
         axutil_generic_obj_set_value(port_obj, env, port_attr);
+        axutil_generic_obj_set_free_func(port_obj, env, axiom_attribute_free_void_arg);
 
         axutil_hash_set(attribute, AXIS2_HTTP_PROXY_HOST, AXIS2_HASH_KEY_STRING,
                         host_obj);
@@ -1582,6 +1485,8 @@ axis2_svc_client_set_proxy_with_auth(
                 password_attr = axiom_attribute_create(env, AXIS2_HTTP_PROXY_PASSWORD, password, NULL);
                 axutil_generic_obj_set_value(username_obj, env, username_attr);
                 axutil_generic_obj_set_value(password_obj, env, password_attr);
+                axutil_generic_obj_set_free_func(username_obj, env, axiom_attribute_free_void_arg);
+                axutil_generic_obj_set_free_func(password_obj, env, axiom_attribute_free_void_arg);
                 axutil_hash_set(attribute, AXIS2_HTTP_PROXY_USERNAME, AXIS2_HASH_KEY_STRING,
                                 username_obj);
                 axutil_hash_set(attribute, AXIS2_HTTP_PROXY_PASSWORD, AXIS2_HASH_KEY_STRING,
