@@ -443,6 +443,8 @@ axis2_module_builder_process_ops(
         else
         {
             op_desc = axis2_op_create_from_module(env);
+            axis2_op_set_msg_exchange_pattern(op_desc, env, mep_url);
+            AXIS2_LOG_DEBUG(env->log, AXIS2_LOG_SI, "mep_url:%s", mep_url); 
         }
 
         op_name = axiom_attribute_get_value(op_name_att, env);
@@ -483,14 +485,18 @@ axis2_module_builder_process_ops(
 
         /* processing <wsp:Policy> .. </..> elements */
 
-        qpolicy =
-            axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
-        itr =
-            axiom_element_get_children_with_qname(op_element, env, qpolicy,
-                                                  op_node);
+        qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
         axutil_qname_free(qpolicy, env);
         qpolicy = NULL;
 
+        if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+        {
+            qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_POLICY_15_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
+            axutil_qname_free(qpolicy, env);
+            qpolicy = NULL;
+        }
         if (itr)
         {
             axis2_process_policy_elements(env, AXIS2_MODULE_OPERATION_POLICY, itr,
@@ -498,14 +504,19 @@ axis2_module_builder_process_ops(
         }
 
         /* processing <wsp:PolicyReference> .. </..> elements */
-        qpolicy =
-            axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
-        itr =
-            axiom_element_get_children_with_qname(op_element, env, qpolicy,
-                                                  op_node);
+        qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
         axutil_qname_free(qpolicy, env);
         qpolicy = NULL;
 
+        if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+        {
+            qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_POLICY_15_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
+            axutil_qname_free(qpolicy, env);
+            qpolicy = NULL;
+        }
+
         if (itr)
         {
             axis2_process_policy_reference_elements(env, AXIS2_POLICY_REF, itr,
