@@ -465,10 +465,16 @@ axis2_svc_builder_populate_svc(
        processing <wsp:Policy> .. </..> elements
      */
     qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
-    itr = axiom_element_get_children_with_qname(svc_element, env,
-                                                qpolicy, svc_node);
+    itr = axiom_element_get_children_with_qname(svc_element, env, qpolicy, svc_node);
     axutil_qname_free(qpolicy, env);
     qpolicy = NULL;
+    if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+    {
+        qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_POLICY_15_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(svc_element, env, qpolicy, svc_node);
+        axutil_qname_free(qpolicy, env);
+        qpolicy = NULL;
+    }
 
     if (itr)
     {
@@ -477,14 +483,18 @@ axis2_svc_builder_populate_svc(
     }
 
     /* processing <wsp:PolicyReference> .. </..> elements */
-    qpolicy =
-        axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
-    itr =
-        axiom_element_get_children_with_qname(svc_element, env, qpolicy,
-                                              svc_node);
+    qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
+    itr = axiom_element_get_children_with_qname(svc_element, env, qpolicy, svc_node);
     axutil_qname_free(qpolicy, env);
     qpolicy = NULL;
 
+    if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+    {
+        qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_POLICY_15_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(svc_element, env, qpolicy, svc_node);
+        axutil_qname_free(qpolicy, env);
+        qpolicy = NULL;
+    }
     if (itr)
     {
         axis2_process_policy_reference_elements(env, AXIS2_POLICY_REF, itr,
@@ -613,7 +623,7 @@ axis2_svc_builder_process_ops(
         else
         {
             axis2_msg_recv_t *msg_recv = NULL;
-            /* setting the default messgae receiver */
+            /* setting the default message receiver */
             msg_recv = axis2_desc_builder_load_default_msg_recv(env);
             axis2_op_set_msg_recv(op_desc, env, msg_recv);
         }
@@ -638,14 +648,19 @@ axis2_svc_builder_process_ops(
 
         /* processing <wsp:Policy> .. </..> elements */
 
-        qpolicy =
-            axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
-        itr =
-            axiom_element_get_children_with_qname(op_element, env, qpolicy,
-                                                  op_node);
+        qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
         axutil_qname_free(qpolicy, env);
         qpolicy = NULL;
 
+        if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+        {
+            qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_POLICY_15_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
+            axutil_qname_free(qpolicy, env);
+            qpolicy = NULL;
+        }
+
         if (itr)
         {
             axis2_process_policy_elements(env, AXIS2_SERVICE_POLICY, itr,
@@ -653,14 +668,19 @@ axis2_svc_builder_process_ops(
         }
 
         /* processing <wsp:PolicyReference> .. </..> elements */
-        qpolicy =
-            axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
-        itr =
-            axiom_element_get_children_with_qname(op_element, env, qpolicy,
-                                                  op_node);
+        qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
+        itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
         axutil_qname_free(qpolicy, env);
         qpolicy = NULL;
 
+        if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+        {
+            qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_POLICY_15_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(op_element, env, qpolicy, op_node);
+            axutil_qname_free(qpolicy, env);
+            qpolicy = NULL;        
+        }
+
         if (itr)
         {
             axis2_process_policy_reference_elements(env, AXIS2_POLICY_REF, itr,
@@ -752,13 +772,18 @@ axis2_svc_builder_process_msgs(
 
             /* processing <wsp:Policy> .. </..> elements */
 
-            qpolicy =
-                axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
-            itr =
-                axiom_element_get_children_with_qname(element, env, qpolicy,
-                                                      node);
+            qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(element, env, qpolicy, node);
             axutil_qname_free(qpolicy, env);
             qpolicy = NULL;
+            
+            if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+            {
+                qpolicy = axutil_qname_create(env, NEETHI_POLICY, NEETHI_POLICY_15_NAMESPACE, NULL);
+                itr = axiom_element_get_children_with_qname(element, env, qpolicy, node);
+                axutil_qname_free(qpolicy, env);
+                qpolicy = NULL;
+            }
 
             if (itr)
             {
@@ -770,15 +795,19 @@ axis2_svc_builder_process_msgs(
             }
 
             /* processing <wsp:PolicyReference> .. </..> elements */
-            qpolicy =
-                axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE,
-                                    NULL);
-            itr =
-                axiom_element_get_children_with_qname(element, env, qpolicy,
-                                                      node);
+            qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_NAMESPACE, NULL);
+            itr = axiom_element_get_children_with_qname(element, env, qpolicy,node);
             axutil_qname_free(qpolicy, env);
             qpolicy = NULL;
 
+            if((!itr) || (!axiom_children_qname_iterator_has_next(itr, env)))
+            {
+                qpolicy = axutil_qname_create(env, NEETHI_REFERENCE, NEETHI_POLICY_15_NAMESPACE, NULL);
+                itr = axiom_element_get_children_with_qname(element, env, qpolicy,node);
+                axutil_qname_free(qpolicy, env);
+                qpolicy = NULL;
+            }
+
             if (itr)
             {
                 axis2_process_policy_reference_elements(env, AXIS2_POLICY_REF,
