@@ -18,44 +18,22 @@
 #ifndef AXIS2_QPID_RECEIVER_H
 #define AXIS2_QPID_RECEIVER_H
 
-#include <qpid/client/Connection.h>
-#include <qpid/client/Dispatcher.h>
-#include <qpid/client/Session.h>
-#include <qpid/client/Message.h>
-#include <qpid/client/MessageListener.h>
 #include <axutil_env.h>
 #include <axis2_conf_init.h>
 
-#include <string>
-
-using namespace qpid::client;
-using namespace qpid::framing;
-using std::string;
-
-class Axis2QpidReceiver : public MessageListener
+class Axis2QpidReceiver
 {
 	public:
-		Axis2QpidReceiver (const axutil_env_t* env,
-						   axis2_conf_ctx_t*   conf_ctx,
-						   string 			   qpidBrokerIP,
-						   int    			   qpidBrokerPort);
-		~Axis2QpidReceiver (void);
+		Axis2QpidReceiver(const axutil_env_t* env,
+						  axis2_conf_ctx_t* conf_ctx);
+		~Axis2QpidReceiver(void);
 
-		bool	start	 (void);
-		bool 	shutdown (void);
+		bool start(void);
+		bool shutdown(void);
 
 	private:
-		virtual void received (Message& message);
-		virtual void listen	  (void);
-		virtual void wait	  (void);
-    
-		const axutil_env_t*   env;
-		axis2_conf_ctx_t*     conf_ctx;
-		string		  		  qpidBrokerIP;
-		int			  		  qpidBrokerPort;
-		Connection    		  connection;
-		Dispatcher*	  		  dispatcher;
-		Session		  		  session;
+		const axutil_env_t* env;
+		axis2_conf_ctx_t*   conf_ctx;
 };
 
 #endif
