@@ -164,6 +164,11 @@ axis2_http_worker_process_request(
     axis2_char_t *url_ext_form = NULL;
     const axis2_char_t *content_type = NULL;
 
+    axis2_msg_ctx_t *out_msg_ctx = NULL;
+    axis2_msg_ctx_t *in_msg_ctx = NULL;
+    axis2_msg_ctx_t **msg_ctx_map = NULL;
+
+
     AXIS2_PARAM_CHECK(env->error, svr_conn, AXIS2_FALSE);
     AXIS2_PARAM_CHECK(env->error, simple_request, AXIS2_FALSE);
 
@@ -517,8 +522,6 @@ axis2_http_worker_process_request(
         is_put = AXIS2_TRUE;
     }
 
-
-        
     request_uri = axis2_http_request_line_get_uri (request_line, env);
     request_params = 
         axis2_http_transport_utils_get_request_params(env,
@@ -1318,8 +1321,8 @@ axis2_http_worker_process_request(
     op_ctx = axis2_msg_ctx_get_op_ctx(msg_ctx, env);
     if (op_ctx)
     {
-        axis2_msg_ctx_t *out_msg_ctx = NULL;
-        axis2_msg_ctx_t **msg_ctx_map = NULL;
+        /*axis2_msg_ctx_t *out_msg_ctx = NULL;
+        axis2_msg_ctx_t **msg_ctx_map = NULL;*/
         axis2_char_t *language_str = NULL;
 
         msg_ctx_map = axis2_op_ctx_get_msg_ctx_map(op_ctx, env);
@@ -1404,16 +1407,6 @@ axis2_http_worker_process_request(
                     char_set = axutil_strltrim(env, char_set, AXIS2_SPACE_TAB_EQ);
                 }
 
-                if (char_set)
-                {
-                    temp2 = strchr(char_set, AXIS2_COMMA);
-                }
-
-                if (temp2)
-                {
-                    *temp2 = AXIS2_ESC_NULL;
-                }
-
                 content_type = axutil_strtrim(env, temp, NULL);
 
                 if (temp)
@@ -1493,8 +1486,19 @@ axis2_http_worker_process_request(
                     AXIS2_FREE(env->allocator, content_type);
                 }
 
+                if (char_set)
+                {
+                    temp2 = strchr(char_set, AXIS2_EQ);
+                }
+
+                if (temp2)
+                {
+                    ++temp2;
+                }
+
                 if (char_set && accept_charset_header_value && 
-                    !axutil_strcasestr(accept_charset_header_value, char_set))
+                    !axutil_strcasestr(accept_charset_header_value, char_set) &&
+                    !axutil_strcasestr(accept_charset_header_value, temp2))
                 {
                     /* 406, Not Acceptable */
                     axis2_http_header_t *cont_len = NULL;
@@ -1598,12 +1602,12 @@ axis2_http_worker_process_request(
             {
                 if (do_rest)
                 {
-                    axis2_msg_ctx_t *out_msg_ctx = NULL;
+                    /*axis2_msg_ctx_t *out_msg_ctx = NULL;
                     axis2_msg_ctx_t *in_msg_ctx = NULL;
-                    axis2_msg_ctx_t **msg_ctx_map = NULL;
+                    axis2_msg_ctx_t **msg_ctx_map = NULL;*/
 
-                    msg_ctx_map = axis2_op_ctx_get_msg_ctx_map(op_ctx, env);
-                    out_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_OUT];
+                    /*msg_ctx_map = axis2_op_ctx_get_msg_ctx_map(op_ctx, env);
+                    out_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_OUT];*/
                     in_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_IN];
                     if (in_msg_ctx)
                     {
@@ -1732,9 +1736,9 @@ axis2_http_worker_process_request(
                 /* If response is not written */
                 if (do_rest)
                 {
-                    axis2_msg_ctx_t *out_msg_ctx = NULL;
+                    /*axis2_msg_ctx_t *out_msg_ctx = NULL;
                     axis2_msg_ctx_t *in_msg_ctx = NULL;
-                    axis2_msg_ctx_t **msg_ctx_map = NULL;
+                    axis2_msg_ctx_t **msg_ctx_map = NULL;*/
 
                     msg_ctx_map = axis2_op_ctx_get_msg_ctx_map(op_ctx, env);
                     out_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_OUT];
@@ -1905,12 +1909,66 @@ axis2_http_worker_process_request(
                 int stream_len = 0;
                 stream_len = axutil_stream_get_len(out_stream, env);
 
-                axis2_http_worker_set_response_headers(http_worker, env, svr_conn,
+                /*axis2_http_worker_set_response_headers(http_worker, env, svr_conn,
                                                        simple_request, response,
-                                                       stream_len);
+                                                       stream_len);*/
    
                 /* This is where it actually write to the wire in the http back channel
                  * append case. */ 
+                if(out_msg_ctx)
+                {
+                    axutil_array_list_t *mime_parts = NULL;
+                    mime_parts = axis2_msg_ctx_get_mime_parts(out_msg_ctx, env);
+                    /* If mime_parts is there then that means we send MTOM. So
+                     * in order to send MTOM we are enabling HTTP1.1 and cunk transfer
+                     * encoding */
+
+                    if(mime_parts)
+                    {
+                        axis2_http_header_t *transfer_enc_header = NULL;   
+                        axutil_param_t *callback_name_param = NULL;
+                        axis2_char_t *mtom_sending_callback_name = NULL;
+
+                        /* Getting the sender callback name paramter if it is 
+                         * specified in the configuration file */
+
+                        callback_name_param = axis2_msg_ctx_get_parameter(out_msg_ctx, env ,
+                            AXIS2_MTOM_SENDING_CALLBACK);
+                        if(callback_name_param)
+                        {
+                            mtom_sending_callback_name =
+                                (axis2_char_t *) axutil_param_get_value (callback_name_param, env);
+                            if(mtom_sending_callback_name)
+                            {
+                                axis2_http_simple_response_set_mtom_sending_callback_name(
+                                    response, env, mtom_sending_callback_name);
+                            }
+                        }
+
+                        axis2_http_simple_response_set_mime_parts(response, env, mime_parts);  
+
+                        axis2_http_simple_response_set_http_version(response, env, 
+                            AXIS2_HTTP_HEADER_PROTOCOL_11);
+ 
+                        transfer_enc_header = axis2_http_header_create(env,
+                                         AXIS2_HTTP_HEADER_TRANSFER_ENCODING,
+                                         AXIS2_HTTP_HEADER_TRANSFER_ENCODING_CHUNKED);
+
+                        axis2_http_simple_response_set_header(response, env,
+                                                  transfer_enc_header);
+
+                        /* In the chunking case content-lenght is zero */
+                        axis2_http_worker_set_response_headers(http_worker, env, svr_conn,
+                                                       simple_request, response,
+                                                       0);
+                    }
+                    else
+                    {
+                        axis2_http_worker_set_response_headers(http_worker, env, svr_conn,
+                                                       simple_request, response,
+                                                       stream_len);
+                    }
+                }
                 status = axis2_simple_http_svr_conn_write_response(svr_conn, 
                                                                    env, 
                                                                    response);
@@ -1944,9 +2002,7 @@ axis2_http_worker_process_request(
 
         if (in_msg_ctx)
         {
-            msg_id =
-                axutil_strdup(env, 
-                              axis2_msg_ctx_get_msg_id(in_msg_ctx, env));
+            msg_id = axutil_strdup(env, axis2_msg_ctx_get_msg_id(in_msg_ctx, env));
             conf_ctx = axis2_msg_ctx_get_conf_ctx(in_msg_ctx, env);
 
             axis2_msg_ctx_free(in_msg_ctx, env);
@@ -1959,10 +2015,7 @@ axis2_http_worker_process_request(
             axis2_op_ctx_destroy_mutex(op_ctx, env);
             if (conf_ctx && msg_id)
             {
-                axis2_conf_ctx_register_op_ctx(conf_ctx, 
-                                               env, 
-                                               msg_id, 
-                                               NULL);
+                axis2_conf_ctx_register_op_ctx(conf_ctx, env, msg_id, NULL);
 
                 AXIS2_FREE(env->allocator, msg_id);
             }
@@ -2050,34 +2103,39 @@ axis2_http_worker_set_response_headers(
                                                           AXIS2_FALSE);
             }
         }
-        if (AXIS2_FALSE ==
-            axis2_http_simple_request_contains_header(simple_request, env,
-                                                      AXIS2_HTTP_HEADER_TRANSFER_ENCODING))
-        {
-            if (0 != content_length)
-            {
-                axis2_char_t content_len_str[10];
-                axis2_http_header_t *content_len_hdr = NULL;
 
-                sprintf(content_len_str, "%d", content_length);
-                content_len_hdr =
-                    axis2_http_header_create(env,
+        if(!axis2_http_simple_response_contains_header(simple_response, env, 
+            AXIS2_HTTP_HEADER_TRANSFER_ENCODING))
+        {
+            if (AXIS2_FALSE ==
+                axis2_http_simple_request_contains_header(simple_request, env,
+                                                      AXIS2_HTTP_HEADER_TRANSFER_ENCODING))
+            {
+                if (0 != content_length)
+                {
+                    axis2_char_t content_len_str[10];
+                    axis2_http_header_t *content_len_hdr = NULL;
+
+                    sprintf(content_len_str, "%d", content_length);
+                    content_len_hdr =
+                        axis2_http_header_create(env,
                                              AXIS2_HTTP_HEADER_CONTENT_LENGTH,
                                              content_len_str);
-                axis2_http_simple_response_set_header(simple_response, env,
+                    axis2_http_simple_response_set_header(simple_response, env,
                                                       content_len_hdr);
+                }
             }
-        }
-        else
-        {
+            else
+            {
             /* Having Transfer encoding Header */
-            axis2_http_header_t *transfer_enc_header =
-                axis2_http_header_create(env,
+                axis2_http_header_t *transfer_enc_header =
+                    axis2_http_header_create(env,
                                          AXIS2_HTTP_HEADER_TRANSFER_ENCODING,
                                          AXIS2_HTTP_HEADER_TRANSFER_ENCODING_CHUNKED);
 
-            axis2_http_simple_response_set_header(simple_response, env,
+                axis2_http_simple_response_set_header(simple_response, env,
                                                   transfer_enc_header);
+            }
         }
     }
     return AXIS2_SUCCESS;
