@@ -34,9 +34,34 @@
 #include <axiom_soap.h>
 #include <axutil_class_loader.h>
 #include <axutil_string_util.h>
+#include <axiom_mime_part.h>
+#include <axiom_mtom_sending_callback.h>
 
 #define READ_SIZE  2048
 
+static axis2_status_t apache2_worker_send_mtom_message(
+    request_rec *request,
+    const axutil_env_t * env,
+    axutil_array_list_t *mime_parts,
+    axis2_char_t *mtom_sending_callback_name);
+
+static axis2_status_t 
+apache2_worker_send_attachment_using_file(
+    const axutil_env_t * env,
+    request_rec *request,
+    FILE *fp,
+    axis2_byte_t *buffer,
+    int buffer_size);
+
+static axis2_status_t
+apache2_worker_send_attachment_using_callback(
+    const axutil_env_t * env,
+    request_rec *request,
+    axiom_mtom_sending_callback_t *callback,
+    void *handler,
+    void *user_param);
+
+
 struct axis2_apache2_worker
 {
     axis2_conf_ctx_t *conf_ctx;
@@ -97,7 +122,8 @@ axis2_apache2_worker_create(
 			hi; hi = axutil_hash_next(env, hi))
 	{
 		void *impl_class = NULL;
-		axutil_param_t *impl_info_param = NULL;
+		axis2_msg_recv_t *msg_recv = NULL;
+		axutil_hash_t *ops_hash = NULL;
 
 		axutil_hash_this(hi, NULL, NULL, &svc);
 		if (!svc)
@@ -105,16 +131,23 @@ axis2_apache2_worker_create(
 		impl_class = axis2_svc_get_impl_class(svc, env);
 		if (impl_class)
 			continue;
-		impl_info_param = axis2_svc_get_param(svc, env, AXIS2_SERVICE_CLASS);
-		if (!impl_info_param)
-			continue;
-
-		axutil_class_loader_init(env);
-		impl_class = axutil_class_loader_create_dll(env, impl_info_param);
-		axis2_svc_set_impl_class(svc, env, impl_class);
-		if (impl_class)
+		ops_hash  = axis2_svc_get_all_ops(svc, env);
+		if(ops_hash)
 		{
-			AXIS2_SVC_SKELETON_INIT((axis2_svc_skeleton_t *) impl_class, env);
+			axutil_hash_index_t *op_hi = NULL;
+			void *op = NULL;
+			op_hi = axutil_hash_first(ops_hash, env);
+			if(op_hi)
+			{
+				axutil_hash_this(op_hi, NULL, NULL, &op);
+				if(op)
+				{
+					msg_recv = axis2_op_get_msg_recv(op,env);
+					if(msg_recv)
+						axis2_msg_recv_load_and_init_svc(msg_recv, env, svc);
+				}
+			}
+
 		}
 	}
 
@@ -173,6 +206,12 @@ axis2_apache2_worker_process_request(
     axis2_char_t *accept_charset_header_value = NULL;
     axis2_char_t *accept_language_header_value = NULL;
     axis2_char_t *content_language_header_value = NULL;
+    axis2_bool_t do_mtom = AXIS2_FALSE;
+    axutil_array_list_t *mime_parts = NULL;
+    axutil_param_t *callback_name_param = NULL;
+    axis2_char_t *mtom_sending_callback_name = NULL;
+    axis2_status_t status = AXIS2_FAILURE;
+
 
     AXIS2_ENV_CHECK(env, AXIS2_CRITICAL_FAILURE);
     AXIS2_PARAM_CHECK(env->error, request, AXIS2_CRITICAL_FAILURE);
@@ -214,6 +253,18 @@ axis2_apache2_worker_process_request(
         content_type = AXIS2_HTTP_HEADER_ACCEPT_TEXT_PLAIN;
     }
     request->content_type = content_type;
+
+    out_desc = axis2_conf_get_transport_out(axis2_conf_ctx_get_conf
+                                            (apache2_worker->conf_ctx, env),
+                                            env, AXIS2_TRANSPORT_ENUM_HTTP);
+    in_desc =
+        axis2_conf_get_transport_in(axis2_conf_ctx_get_conf
+                                    (apache2_worker->conf_ctx, env), env,
+                                    AXIS2_TRANSPORT_ENUM_HTTP);
+
+    msg_ctx = axis2_msg_ctx_create(env, conf_ctx, in_desc, out_desc);
+    axis2_msg_ctx_set_server_side(msg_ctx, env, AXIS2_TRUE);
+
     if (request->read_chunked == AXIS2_TRUE && 0 == content_length)
     {
         content_length = -1;
@@ -229,17 +280,6 @@ axis2_apache2_worker_process_request(
     AXIS2_LOG_DEBUG(env->log, AXIS2_LOG_SI, "Client HTTP version %s",
                     http_version);
 
-    out_desc = axis2_conf_get_transport_out(axis2_conf_ctx_get_conf
-                                            (apache2_worker->conf_ctx, env),
-                                            env, AXIS2_TRANSPORT_ENUM_HTTP);
-    in_desc =
-        axis2_conf_get_transport_in(axis2_conf_ctx_get_conf
-                                    (apache2_worker->conf_ctx, env), env,
-                                    AXIS2_TRANSPORT_ENUM_HTTP);
-
-    msg_ctx = axis2_msg_ctx_create(env, conf_ctx, in_desc, out_desc);
-    axis2_msg_ctx_set_server_side(msg_ctx, env, AXIS2_TRUE);
-
     peer_ip = request->connection->remote_ip;
     
     if (peer_ip)
@@ -613,7 +653,7 @@ axis2_apache2_worker_process_request(
     }
     else if (M_POST == request->method_number || M_PUT == request->method_number)
     {
-        axis2_status_t status = AXIS2_FAILURE;
+        /*axis2_status_t status = AXIS2_FAILURE;*/
         if (M_POST == request->method_number)
         {
             status = axis2_http_transport_utils_process_http_post_request
@@ -1159,6 +1199,31 @@ axis2_apache2_worker_process_request(
         out_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_OUT];
         in_msg_ctx = msg_ctx_map[AXIS2_WSDL_MESSAGE_LABEL_IN];
 
+        /* In mtom case we send the attachment differently */
+
+        /* status = AXIS2_FAILURE means fault scenario. We are not 
+         * doing MTOM for fault cases. */
+
+        if(status != AXIS2_FAILURE)
+        {
+            do_mtom = axis2_msg_ctx_get_doing_mtom(out_msg_ctx, env);
+            if(do_mtom)
+            {
+                mime_parts = axis2_msg_ctx_get_mime_parts(out_msg_ctx, env);
+                if(!mime_parts)
+                {
+                    return AXIS2_FAILURE;
+                }
+                callback_name_param = axis2_msg_ctx_get_parameter(msg_ctx, env ,
+                    AXIS2_MTOM_SENDING_CALLBACK);
+                if(callback_name_param)
+                {
+                    mtom_sending_callback_name =
+                        (axis2_char_t *) axutil_param_get_value (callback_name_param, env);
+                }            
+            }
+        }
+
         if (out_msg_ctx)
         {
             axis2_msg_ctx_free(out_msg_ctx, env);
@@ -1189,7 +1254,46 @@ axis2_apache2_worker_process_request(
         }
 
     }                           /* Done freeing message contexts */
-    if (body_string)
+
+    /* We send the message in parts when doing MTOM */
+
+    if(do_mtom)
+    {
+        axis2_status_t mtom_status = AXIS2_FAILURE;
+
+        if(!mtom_sending_callback_name)
+        {
+            /* If the callback name is not there, then we will check whether there 
+             * is any mime_parts which has type callback. If we found then no point 
+             * of continuing we should return a failure */
+
+            if(!mtom_sending_callback_name)
+            {
+                if(axis2_http_transport_utils_is_callback_required(
+                    env, mime_parts))
+                {
+                    AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "Sender callback not specified");
+                    return AXIS2_FAILURE;
+                }
+            }
+        }
+
+        mtom_status = apache2_worker_send_mtom_message(request, env, mime_parts, 
+            mtom_sending_callback_name);
+        if(mtom_status == AXIS2_SUCCESS)
+        {
+            send_status = DONE;
+        }
+        else
+        {
+            send_status = DECLINED;
+        }
+
+        axis2_http_transport_utils_destroy_mime_parts(mime_parts, env);
+        mime_parts = NULL;
+    }
+
+    else if (body_string)
     {
         ap_rwrite(body_string, body_string_len, request);
         body_string = NULL;
@@ -1252,3 +1356,235 @@ axis2_apache2_worker_get_bytes(
     axutil_stream_free(tmp_stream, env);
     return buffer;
 }
+
+
+static axis2_status_t apache2_worker_send_mtom_message(
+    request_rec *request,
+    const axutil_env_t * env,
+    axutil_array_list_t *mime_parts,
+    axis2_char_t *mtom_sending_callback_name)
+{
+    int i = 0;
+    axiom_mime_part_t *mime_part = NULL;
+    axis2_status_t status = AXIS2_SUCCESS;
+    /*int written = 0;*/
+    int len = 0;    
+
+    if(mime_parts)
+    {
+        for(i = 0; i < axutil_array_list_size
+                (mime_parts, env); i++)
+        {
+            mime_part = (axiom_mime_part_t *)axutil_array_list_get(
+                mime_parts, env, i);
+            if((mime_part->type) == AXIOM_MIME_PART_BUFFER)
+            {
+                len = 0;
+                len = ap_rwrite(mime_part->part, mime_part->part_size, request);
+                ap_rflush(request);
+                if(len == -1)
+                {
+                    status = AXIS2_FAILURE;
+                    break;
+                }
+            }
+            else if((mime_part->type) == AXIOM_MIME_PART_FILE)
+            {
+                FILE *f = NULL;
+                axis2_byte_t *output_buffer = NULL;                
+                int output_buffer_size = 0;
+
+                f = fopen(mime_part->file_name, "rb");
+                if (!f)
+                {
+                    AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                        "Error opening file %s for reading",
+                    mime_part->file_name);
+                    return AXIS2_FAILURE;
+                }
+                if(mime_part->part_size > AXIS2_MTOM_OUTPUT_BUFFER_SIZE)
+                {
+                    output_buffer_size = AXIS2_MTOM_OUTPUT_BUFFER_SIZE;
+                }
+                else
+                {
+                    output_buffer_size = mime_part->part_size;
+                }
+               
+                output_buffer =  AXIS2_MALLOC(env->allocator, 
+                    (output_buffer_size + 1) * sizeof(axis2_char_t));
+ 
+ 
+                status = apache2_worker_send_attachment_using_file(env, request, 
+                    f, output_buffer, output_buffer_size);
+                if(status == AXIS2_FAILURE)
+                {
+                    return status;
+                }
+            }
+            else if((mime_part->type) == AXIOM_MIME_PART_CALLBACK)
+            {
+                void *handler = NULL;
+                axiom_mtom_sending_callback_t *callback = NULL;
+
+                handler = axis2_http_transport_utils_initiate_callback(env,
+                    mtom_sending_callback_name, mime_part->user_param, &callback);
+               
+                if(handler)
+                {
+                    status = apache2_worker_send_attachment_using_callback(env,
+                        request, callback, handler, mime_part->user_param);
+                }
+                else
+                {
+                    AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI, "MTOM Sending Callback loading failed");
+                    status = AXIS2_FAILURE;
+                }
+                if(callback)
+                {
+                    axutil_param_t *param = NULL;
+
+                    param = callback->param;
+
+                    AXIOM_MTOM_SENDING_CALLBACK_FREE(callback, env);
+                    callback = NULL;
+
+                    if(param)
+                    {
+                        axutil_param_free(param, env);
+                        param = NULL;
+                    }
+
+                }
+
+                if(status == AXIS2_FAILURE)
+                {
+                    return status;
+                }
+            }
+            else
+            {
+                return AXIS2_FAILURE;
+            }
+            if(status == AXIS2_FAILURE)
+            {
+                break;
+            }
+        }
+        return status;
+    }    
+    else
+    {
+        return AXIS2_FAILURE;
+    }    
+}
+
+
+static axis2_status_t
+apache2_worker_send_attachment_using_file(
+    const axutil_env_t * env,
+    request_rec *request,
+    FILE *fp,
+    axis2_byte_t *buffer,
+    int buffer_size)
+{
+
+    int count = 0;     
+    int len = 0;
+    /*int written = 0;*/
+    axis2_status_t status = AXIS2_SUCCESS;   
+ 
+    do
+    {
+        count = (int)fread(buffer, 1, buffer_size + 1, fp);
+        if (ferror(fp))
+        {
+            AXIS2_LOG_ERROR(env->log, AXIS2_LOG_SI,
+                "Error in reading file containg the attachment");
+            if (buffer)
+            {
+                AXIS2_FREE(env->allocator, buffer);
+                buffer = NULL;
+            }
+            fclose(fp);
+            return AXIS2_FAILURE;
+        }
+
+        if(count > 0)
+        {
+            len = 0;
+            len = ap_rwrite(buffer, count, request);
+            ap_rflush(request);
+            if(len == -1)
+            {
+                status = AXIS2_FAILURE;
+                break;
+            }
+        }
+        else
+        {
+            if (buffer)
+            {
+                AXIS2_FREE(env->allocator, buffer);
+                buffer = NULL;
+            }
+            fclose(fp);
+            return AXIS2_FAILURE;
+        }   
+        memset(buffer, 0, buffer_size);    
+        if(status == AXIS2_FAILURE)
+        {
+            if (buffer)
+            {
+                AXIS2_FREE(env->allocator, buffer);
+                buffer = NULL;
+            }
+            fclose(fp);
+            return AXIS2_FAILURE;
+        } 
+    }
+    while(!feof(fp));
+    
+    fclose(fp);
+    AXIS2_FREE(env->allocator, buffer);
+    buffer = NULL;    
+    return AXIS2_SUCCESS;    
+}
+
+static axis2_status_t
+apache2_worker_send_attachment_using_callback(
+    const axutil_env_t * env,
+    request_rec *request,
+    axiom_mtom_sending_callback_t *callback,
+    void *handler,
+    void *user_param)
+{
+    int count = 0;
+    int len = 0;
+    axis2_status_t status = AXIS2_SUCCESS;
+    axis2_char_t *buffer = NULL;
+
+    /* Keep on loading the data in a loop until 
+     * all the data is sent */
+
+    while((count = AXIOM_MTOM_SENDING_CALLBACK_LOAD_DATA(callback, env, handler, &buffer)) > 0)
+    {
+        len = 0;
+        len = ap_rwrite(buffer, count, request);
+        ap_rflush(request);
+        if(len == -1)
+        {
+            status = AXIS2_FAILURE;
+            break;
+        }
+    }
+
+    if (status == AXIS2_FAILURE)
+    {
+        AXIOM_MTOM_SENDING_CALLBACK_CLOSE_HANDLER(callback, env, handler);
+        return status;
+    }
+
+    status = AXIOM_MTOM_SENDING_CALLBACK_CLOSE_HANDLER(callback, env, handler);
+    return status;
+}
