@@ -105,10 +105,6 @@ void AXIS2_CALL axis2_module_free(
     axutil_allocator_t * allocator,
     void *ptr);
 
-void AXIS2_CALL
-axis2_module_pool_allocator_destroy(
-    axutil_allocator_t *allocator);
-
 static void axis2_module_init(
     apr_pool_t * p,
     server_rec * svr_rec);
@@ -323,16 +319,15 @@ axis2_handler(
     axutil_env_t *thread_env = NULL;
     axutil_allocator_t *allocator = NULL;
     axutil_error_t *error = NULL;
+
     apr_allocator_t *local_allocator = NULL;
     apr_pool_t *local_pool = NULL;
 
-    apr_allocator_create(&local_allocator);
-    apr_pool_create_ex(&local_pool, NULL, NULL, local_allocator);
-
     if (strcmp(req->handler, "axis2_module"))
     {
         return DECLINED;
     }
+
     /* Set up the read policy from the client. */
     if ((rv = ap_setup_client_block(req, REQUEST_CHUNKED_DECHUNK)) != OK)
     {
@@ -340,6 +335,10 @@ axis2_handler(
     }
     ap_should_client_block(req);
 
+    apr_allocator_create(&local_allocator);
+    apr_pool_create_ex(&local_pool, NULL, NULL, local_allocator);
+
+    
 
     /*thread_env = axutil_init_thread_env(axutil_env);*/
 
@@ -347,8 +346,12 @@ axis2_handler(
     rv = AXIS2_APACHE2_WORKER_PROCESS_REQUEST(axis2_worker, axutil_env, req);*/
 
     /* create new allocator for this request */
+    /*allocator = (axutil_allocator_t *) apr_palloc(req->pool,
+                                                  sizeof(axutil_allocator_t));*/
+
     allocator = (axutil_allocator_t *) apr_palloc(local_pool,
                                                   sizeof(axutil_allocator_t));
+
     if (!allocator)
     {
         return HTTP_INTERNAL_SERVER_ERROR;
@@ -356,12 +359,9 @@ axis2_handler(
     allocator->malloc_fn = axis2_module_malloc;
     allocator->realloc = axis2_module_realloc;
     allocator->free_fn = axis2_module_free;
-    allocator->local_pool = (void *) local_pool;
-    allocator->current_pool = (void *) local_pool;
+    allocator->local_pool = (void *)local_pool;
+    allocator->current_pool = (void *)local_pool;
     allocator->global_pool = axutil_env->allocator->global_pool;
-    allocator->pool_allocator_destroy = axis2_module_pool_allocator_destroy;
-    allocator->pool_allocator = local_allocator;
-    allocator->ref_pool_allocator = 1; 
 
     error = axutil_error_create(allocator);
     thread_env = axutil_env_create_with_error_log_thread_pool(allocator,
@@ -376,10 +376,9 @@ axis2_handler(
         return HTTP_INTERNAL_SERVER_ERROR;
     }
 
-    axis2_module_pool_allocator_destroy(allocator);    
+    apr_pool_destroy(local_pool);
+    apr_allocator_destroy(local_allocator);
 
-    /*apr_pool_destroy(local_pool);
-    apr_allocator_destroy(local_allocator);*/
 
     return rv;
 }
@@ -446,18 +445,6 @@ axis2_module_free(
 #endif
 }
 
-void AXIS2_CALL 
-axis2_module_pool_allocator_destroy(
-    axutil_allocator_t *allocator)
-{
-    if((--(allocator->ref_pool_allocator)) == 0)
-    {
-        apr_pool_destroy(allocator->local_pool);
-        apr_allocator_destroy(allocator->pool_allocator);
-    }
-}
-
-
 static int axis2_post_config(apr_pool_t *pconf, apr_pool_t *plog,
 								 apr_pool_t *ptemp, server_rec *svr_rec)
 {
@@ -713,7 +700,7 @@ axis2_module_init(
         exit(APEXIT_CHILDFATAL);
     }
 	/* If we are initialized we register a clean up as well */
-	/* apr_pool_cleanup_register(p, NULL, axis2_shutdown, apr_pool_cleanup_null); */
+	/* apr_pool_cleanup_register(p, NULL, axis2_shutdown, apr_pool_cleanup_null);*/
 }
 
 static void
