@@ -691,6 +691,22 @@ axutil_date_time_serialize_date_time(
     return date_time_str;
 }
 
+AXIS2_EXTERN axis2_char_t *AXIS2_CALL
+axutil_date_time_serialize_date_time_without_millisecond(
+    axutil_date_time_t *date_time,
+    const axutil_env_t *env)
+{
+    axis2_char_t *date_time_str = NULL;
+
+    AXIS2_ENV_CHECK(env, NULL);
+
+    date_time_str = AXIS2_MALLOC(env->allocator, sizeof(char) * 32);
+    sprintf(date_time_str, "%d-%02d-%02dT%02d:%02d:%02.0fZ",
+        date_time->year + 1900, date_time->mon + 1, date_time->day,
+        date_time->hour, date_time->min, date_time->sec);
+    return date_time_str;
+}
+
 AXIS2_EXTERN axis2_char_t *AXIS2_CALL
 axutil_date_time_serialize_date_time_with_time_zone(
     axutil_date_time_t *date_time,
