@@ -342,6 +342,7 @@ axutil_hash_copy(
              sizeof(axutil_hash_t) + sizeof(*ht->array) * (orig->max + 1) +
              sizeof(axutil_hash_entry_t) * orig->count);
     ht->env = env;
+    axutil_env_increment_ref((axutil_env_t*)env);
     ht->free = NULL;
     ht->count = orig->count;
     ht->max = orig->max;
@@ -451,7 +452,7 @@ axutil_hash_merge(
     axutil_hash_entry_t *new_vals = NULL;
     axutil_hash_entry_t *iter;
     axutil_hash_entry_t *ent;
-    unsigned int i, j, k;
+    unsigned int i, k;
 
 #if AXIS2_POOL_DEBUG
     /* we don't copy keys and values, so it's necessary that
@@ -474,6 +475,7 @@ axutil_hash_merge(
 
     res = AXIS2_MALLOC(env->allocator, sizeof(axutil_hash_t));
     res->env = env;
+    axutil_env_increment_ref((axutil_env_t*)env);
     res->free = NULL;
     res->hash_func = base->hash_func;
     res->count = base->count;
@@ -483,25 +485,18 @@ axutil_hash_merge(
         res->max = res->max * 2 + 1;
     }
     res->array = axutil_hash_alloc_array(res, res->max);
-    if (base->count + overlay->count)
-    {
-        new_vals = AXIS2_MALLOC(env->allocator,
-                       sizeof(axutil_hash_entry_t) * (base->count +
-                       overlay->count));
-    }
-    j = 0;
     for (k = 0; k <= base->max; k++)
     {
         for (iter = base->array[k]; iter; iter = iter->next)
         {
             i = iter->hash & res->max;
-            new_vals[j].klen = iter->klen;
-            new_vals[j].key = iter->key;
-            new_vals[j].val = iter->val;
-            new_vals[j].hash = iter->hash;
-            new_vals[j].next = res->array[i];
-            res->array[i] = &new_vals[j];
-            j++;
+            new_vals = AXIS2_MALLOC(env->allocator, sizeof(axutil_hash_entry_t));
+            new_vals->klen = iter->klen;
+            new_vals->key = iter->key;
+            new_vals->val = iter->val;
+            new_vals->hash = iter->hash;
+            new_vals->next = res->array[i];
+            res->array[i] = new_vals;
         }
     }
 
@@ -530,14 +525,14 @@ axutil_hash_merge(
             }
             if (!ent)
             {
-                new_vals[j].klen = iter->klen;
-                new_vals[j].key = iter->key;
-                new_vals[j].val = iter->val;
-                new_vals[j].hash = iter->hash;
-                new_vals[j].next = res->array[i];
-                res->array[i] = &new_vals[j];
+                new_vals = AXIS2_MALLOC(env->allocator, sizeof(axutil_hash_entry_t));
+                new_vals->klen = iter->klen;
+                new_vals->key = iter->key;
+                new_vals->val = iter->val;
+                new_vals->hash = iter->hash;
+                new_vals->next = res->array[i];
+                res->array[i] = new_vals;
                 res->count++;
-                j++;
             }
         }
     }
@@ -566,6 +561,7 @@ axutil_hash_contains_key(
     return AXIS2_FALSE;
 }
 
+/*
 void
 axutil_hash_entry_free(
     const axutil_env_t *env,
@@ -580,6 +576,7 @@ axutil_hash_entry_free(
     AXIS2_FREE(env->allocator, hash_entry);
     return;
 }
+*/
 
 AXIS2_EXTERN void AXIS2_CALL
 axutil_hash_free(
@@ -665,7 +662,18 @@ axutil_hash_set_env(
     AXIS2_ENV_CHECK(env, AXIS2_FAILURE);
     if (ht)
     {
+        if (ht->env)
+        {
+            /*since we now keep a ref count in env and incrementing it
+             *inside hash_make we need to free the env.Depending on the
+              situation the env struct is freed or ref count will be 
+              decremented.*/
+
+            axutil_free_thread_env((axutil_env_t*)(ht->env));
+            ht->env = NULL;
+        }
         ht->env = env;
+        axutil_env_increment_ref((axutil_env_t*)env);
     }
 }
 
